package _mapping_consistency;

import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.util.EList;

import ronmodel.Codomain;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RON;
import ronmodel.Rule;
import ronmodel.Token;

public class MappingConsistencyCheck {
	
	
//	/**
//	 * calculates the mapping number of "mapping"
//	 * @param mapping a mapping
//	 * @return mappingnr of mapping
//	 */
//	private static int getMappingNr(Mapping mapping)
//	throws NullPointerException{
//		int i=-2;
//		for (Object o: mapping.getSource().eAdapters()){
//			if (o instanceof ONNodeEditPart){
//				ONNodeEditPart onep=(ONNodeEditPart)o;
//				RulePage rp= ((RulePage)((MultiViewerPageViewer)onep.getViewer()).getHostPage());
//				MappingNumberList mnl=rp.mappingNumberList;
//				i=mnl.indexOfMapping(mapping);
//				break;
//			}
//		}		
//		return i+1; //-1 if there is no mapping nr !
//	}
	
	
	/**
	 * Checks the pre-condition for given mapping (and changes result, if it is violated)
	 * ## Only works correct for injective mapping of pre- and post-places. ##
	 * @param mapping Mapping to check
	 * @param result MappingConsistencyResult to change if pre-condition is violated
	 */
	private static void checkPreCondition(Mapping mapping, MappingConsistencyResult result)
	throws NullPointerException{		
		ONNode source=(ONNode)mapping.getSource();
		ONNode target=(ONNode)mapping.getTarget(); 
		if (source instanceof ONPlace) return;
		if (target instanceof ONPlace) return;
		Codomain codomain=mapping.getCodomain();
		EList incomingS=source.getIncoming();
		EList incomingT=target.getIncoming();
		//build vector with all preNodes of target
		Vector<ONNode> preNodesTarget=new Vector<ONNode>();
		Vector<Integer> preArcsTarget=new Vector<Integer>();
		for (Object o1: incomingT){
			//for all incoming arcs of target
			preNodesTarget.add(((ONArc)o1).getSource());
			preArcsTarget.add(((ONArc)o1).getWeight());			
		}				
		for (Object o1: incomingS){
			//for all incoming arcs of source
			ONNode on1=((ONArc)o1).getSource();
			EList on1Mappings=on1.getOutgoingM();
			boolean mapped=false;
			for (Object o2: on1Mappings){
				//for all outgoing mappings of source				
				Mapping on1Mapping=(Mapping)o2;
				if (on1Mapping.getCodomain()==codomain){
					//for all outgoing mappings of source with right codomain
					mapped=true;
					if (!preNodesTarget.contains(on1Mapping.getTarget())){
						result.setResult(MappingConsistencyResult.PRE_COND_VIOLATED);						
						result.setMessageWithNames("Violated pre-condition for target node '"+target.getName()+"' (can not find codomain of source node '"+on1.getName()+"')!");
						result.setMessageWithMappingNrs("Violated pre-condition for mapping nr '"+mapping.getMappingNr()+"' (can not find codomain of mapping nr '"+on1Mapping.getMappingNr()+"')!");
						result.setMapping(mapping);
						result.setMappingNr(mapping.getMappingNr());
						result.setErrorNode(on1);
						return;
					}
					int index=preNodesTarget.indexOf(on1Mapping.getTarget());
					int targetWeight=preArcsTarget.get(index);
					if (targetWeight!=((ONArc)o1).getWeight()){
						result.setResult(MappingConsistencyResult.PRE_COND_VIOLATED);
						result.setMessageWithNames("Violated pre-condition for target node '"+target.getName()+"' (wrong weight of target-arc from '"+(on1Mapping.getTarget().getName())+"'))!");
						result.setMessageWithMappingNrs("Violated pre-condition for mapping nr '"+mapping.getMappingNr()+"' (wrong weight of target-arc from mapping nr '"+on1Mapping.getMappingNr()+"')!");
						result.setMapping(mapping);
						result.setMappingNr(mapping.getMappingNr());
						result.setErrorNode(on1);
						return;
					}
					preNodesTarget.remove(index);
					preArcsTarget.remove(index);					
				}
			}
			if (!mapped){
				//there is no codomain for this node (and no mappings)
				result.setResult(MappingConsistencyResult.PRE_COND_VIOLATED);
				result.setMessageWithNames("Violated pre-condition for target node '"+target.getName()+"' (can not find codomain of source node '"+on1.getName()+"')!");
				result.setMessageWithMappingNrs("Violated pre-condition for mapping nr '"+mapping.getMappingNr()+"' (can not find codomain of a node)!");
				result.setMapping(mapping);
				result.setMappingNr(mapping.getMappingNr());
				result.setErrorNode(on1);
				return;
			}
		}
		if (!preNodesTarget.isEmpty()){			
			result.setResult(MappingConsistencyResult.PRE_COND_VIOLATED);
			result.setMessageWithNames("Violated pre-condition for target node '"+target.getName()+"' (can not find domain of target node '"+preNodesTarget.get(0).getName()+"')!");
			result.setMessageWithMappingNrs("Violated pre-condition for mapping nr '"+mapping.getMappingNr()+"' (can not find domain of a node)!");
			result.setMapping(mapping);
			result.setMappingNr(mapping.getMappingNr());
			result.setErrorNode(preNodesTarget.get(0));
			return;
		}
	}
	
	/**
	 * Checks the post-condition for given mapping (and changes result, if it is violated)
	 * ## Only works correct for injective mapping of pre- and post-places. ##
	 * @param mapping Mapping to check
	 * @param result MappingConsistencyResult to change if post-condition is violated
	 */
	private static void checkPostCondition(Mapping mapping, MappingConsistencyResult result)
	throws NullPointerException{		
		ONNode source=(ONNode)mapping.getSource();
		ONNode target=(ONNode)mapping.getTarget(); 
		if (source instanceof ONPlace) return;
		if (target instanceof ONPlace) return;
		Codomain codomain=mapping.getCodomain();
		EList outgoingS=source.getOutgoing();
		EList outgoingT=target.getOutgoing();
		//build vector with all postNodes of target
		Vector<ONNode> postNodesTarget=new Vector<ONNode>();
		Vector<Integer> postArcsTarget=new Vector<Integer>();
		for (Object o1: outgoingT){
			//for all outgoing arcs of target
			postNodesTarget.add(((ONArc)o1).getTarget());
			postArcsTarget.add(((ONArc)o1).getWeight());
		}				
		for (Object o1: outgoingS){
			//for all outgoing arcs of source
			ONNode on1=((ONArc)o1).getTarget();
			EList on1Mappings=on1.getOutgoingM();
			boolean mapped=false;
			for (Object o2: on1Mappings){
				//for all outgoing mappings of source				
				Mapping on1Mapping=(Mapping)o2;
				if (on1Mapping.getCodomain()==codomain){
					//for all outgoing mappings of source with right codomain	
					mapped=true;
					if (!postNodesTarget.contains(on1Mapping.getTarget())){
						result.setResult(MappingConsistencyResult.POST_COND_VIOLATED);
						result.setMessageWithNames("Violated post-condition for target node '"+target.getName()+"' (can not find codomain of source node '"+on1.getName()+"')!");
						result.setMessageWithMappingNrs("Violated post-condition for mapping nr '"+mapping.getMappingNr()+"' (can not find codomain of mapping nr '"+on1Mapping.getMappingNr()+"')!");
						result.setMapping(mapping);
						result.setMappingNr(mapping.getMappingNr());
						result.setErrorNode(on1);
						return;
					}
					int index=postNodesTarget.indexOf(on1Mapping.getTarget());
					int targetWeight=postArcsTarget.get(index);
					if (targetWeight!=((ONArc)o1).getWeight()){
						result.setResult(MappingConsistencyResult.POST_COND_VIOLATED);
						result.setMessageWithNames("Violated post-condition for target node '"+target.getName()+"' (wrong weight of target-arc to '"+(on1Mapping.getTarget().getName())+"')!");
						result.setMessageWithMappingNrs("Violated post-condition for mapping nr '"+mapping.getMappingNr()+"' (wrong weight of target-arc to mapping nr '"+on1Mapping.getMappingNr()+"')!");
						result.setMapping(mapping);
						result.setMappingNr(mapping.getMappingNr());
						result.setErrorNode(on1);
						return;
					}					
					postNodesTarget.remove(index);
					postArcsTarget.remove(index);					
				}
			}
			if (!mapped){
				//there is no codomain for this node (and no mappings)
				result.setResult(MappingConsistencyResult.POST_COND_VIOLATED);
				result.setMessageWithNames("Violated post-condition for target node '"+target.getName()+"' (can not find codomain of source node '"+on1.getName()+"')!");
				result.setMessageWithMappingNrs("Violated post-condition for mapping nr '"+mapping.getMappingNr()+"' (can not find codomain of a node)!");
				result.setMapping(mapping);
				result.setMappingNr(mapping.getMappingNr());
				result.setErrorNode(on1);
				return;
			}
		}				
		if (!postNodesTarget.isEmpty()){
			result.setResult(MappingConsistencyResult.POST_COND_VIOLATED);
			result.setMessageWithNames("Violated post-condition for target node '"+target.getName()+"' (can not find domain of target node '"+postNodesTarget.get(0).getName()+"')!");
			result.setMessageWithMappingNrs("Violated post-condition for mapping nr '"+mapping.getMappingNr()+"' (can not find domain of a node)!");
			result.setMapping(mapping);
			result.setMappingNr(mapping.getMappingNr());
			result.setErrorNode(postNodesTarget.get(0));
			return;
		}
	}
	
	/**
	 * Checks the consistency of all mappings of given codomain.
	 * Checks mapping for
	 *  - type consistency
	 * 	- left totality
	 * 	- injectivity
	 * 	- pre-conditions (of all transitions)
	 * 	- post-conditions (of all transitions)
	 * 
	 * @param codomain Codomain
	 * @return MappingConsistencyResult with many informations about the mapping (see MappingConsistencyResult for more informations)
	 */
	private static MappingConsistencyResult checkMappingConsistency(Codomain codomain){
		MappingConsistencyResult result=new MappingConsistencyResult(codomain);
		try{					
			Vector<ONNode> alreadyMappedTo=new Vector<ONNode>(); //for injectivity-check
			Vector<ONNode> alreadyMappedFrom=new Vector<ONNode>(); //for left-total-check
			ObjectNet domain=null;
			
			EList mappings=codomain.getMappings();
			if (mappings.isEmpty()){
				return result; //return if there is no mapping (no mapping = consistent)
			}
			for (Object o: mappings){			
				Mapping mapping=(Mapping)o;
				domain=mapping.getSource().getObjectNet(); //there is only one domain for any codomain
				ONNode source=mapping.getSource();
				ONNode target=mapping.getTarget();
				
				//check place to place
				if (source instanceof ONPlace && !(target instanceof ONPlace)){				
					result.setResult(MappingConsistencyResult.TYPES_VIOLATED);
					result.setMessageWithNames("Can only map places to places (see source node '"+source.getName()+"')!");
					result.setMessageWithMappingNrs("Can only map places to places (see mapping nr '"+mapping.getMappingNr()+"')!");					
					result.setMapping(mapping);
					result.setMappingNr(mapping.getMappingNr());
					result.setErrorNode(source);
					return result;
				}
				
				//check trans to trans
				if (source instanceof ONTransition && !(target instanceof ONTransition)){
					result.setResult(MappingConsistencyResult.TYPES_VIOLATED);
					result.setMessageWithNames("Can only map transitions to transitions (see source node '"+source.getName()+"')!");
					result.setMessageWithMappingNrs("Can only map transitions to transitions (see mapping nr '"+mapping.getMappingNr()+"')!");
					result.setMapping(mapping);
					result.setMappingNr(mapping.getMappingNr());
					result.setErrorNode(source);
					return result;
				}
				
				//check pre
				if (source instanceof ONTransition){
					checkPreCondition(mapping,result);
				}
				//check post
				if (source instanceof ONTransition){
					checkPostCondition(mapping,result);				
				}
				
				//check injectivity
				if (alreadyMappedTo.contains(target)){
					//mapping is not injective
					result.setResult(MappingConsistencyResult.INJECTIVITY_VIOLATED);
					result.setMessageWithNames("Only injective mappings are allowed (see source node: '"+source.getName()+"')!");
					result.setMessageWithMappingNrs("Only injective mappings are allowed (see mappingNr: '"+mapping.getMappingNr()+"')!");
					result.setMapping(mapping);
					result.setMappingNr(mapping.getMappingNr());
					result.setErrorNode(source);
					return result;
				} alreadyMappedTo.add(target);
				
				//prepare vector for check left-total
				alreadyMappedFrom.add(source);
							
			}
			if (!result.isConsistent()) return result; // return result if mapping is inconsistent
			
			//check left-total
			for (Object o: domain.getNodes()){
				ONNode onNode=(ONNode)o;
				if (!alreadyMappedFrom.contains(o)){
					result.setResult(MappingConsistencyResult.LEFT_TOTALITY_VIOLATED);
					result.setMessageWithNames("Mapping is consistent but not left-total (source node: '"+onNode.getName()+"' has no codomain)!");
					result.setMessageWithMappingNrs("Mapping is consistent but not left-total (a node has no codomain)!");					
					result.setErrorNode(onNode);
					return result;
				}
			}			
		} catch (NullPointerException npe){
			result.setMessageWithNames("** INTERNAL STRUCTURE FAULT (USERS SHOULD NOT SEE THIS) **");
			result.setMessageWithMappingNrs("** INTERNAL STRUCTURE FAULT (USERS SHOULD NOT SEE THIS) **");
			result.setResult(MappingConsistencyResult.INTERNAL_STRUCTURE_FAULT);
		}
		return result;		
	}
	
	/**
	 * Checks mapping consistency for given objectnet 
	 * if the objectnet is a codomain itself,
	 * otherwise checks mapping consistency for all
	 * related codomains (on has to be be part of a rule)).
	 * Checks mapping for RULES! (LHS->RHS, LHS->NAC) 
	 * Sets markers if the mapping is inconsistent.
	 * @param on ObjectNet of a RULE (LHS, RHS or NAC)
	 * @param res the resource for the markers 
	 */
	public static void checkMappingConsistency(ObjectNet on, IResource resource){		
		if (on==null) return;		
		if (on instanceof Codomain){
			//check for on
			MappingConsistencyResult mcr=MappingConsistencyCheck.checkMappingConsistency((Codomain)on);
			if (!mcr.internalStructureFault()){
				deleteAllMarkers(resource,mcr);
				if (!mcr.isConsistent())
					setMarker(resource, mcr);
			}
		} else
		if (on instanceof LHS){
			//check for all codomains of on				
			Rule rule=((LHS)on).getRule();
			for (Object o: rule.getNacs()){				
				Codomain codom=(Codomain)o;
				MappingConsistencyResult mcr=MappingConsistencyCheck.checkMappingConsistency(codom);
				if (!mcr.internalStructureFault()) {
					deleteAllMarkers(resource,mcr);
					if (!mcr.isConsistent())
						setMarker(resource, mcr);				
				}
			}
			MappingConsistencyResult mcr=MappingConsistencyCheck.checkMappingConsistency(rule.getRhs());
			if (!mcr.internalStructureFault()){
				deleteAllMarkers(resource,mcr);
				if (!mcr.isConsistent())
					setMarker(resource, mcr);			
			}
		}
	}
	
	/**
	 * Does a consistency check for all rules of the whole RON. Deletes all old markers and creates new ones.
	 * @param res resource
	 * @param ron the ron
	 */
	public static void checkMappingConsistency(RON ron, IResource res){
		deleteAllMarkers(res);
		ArrayList<Rule> rules=getAllRules(ron);
		for (Rule r: rules){
			checkMappingConsistency(r.getLhs(),res);
		}
	}

	/**
	 * Checks mapping-consistency of given ron for all rules that have no markers at the moment.
	 * @param res resource
	 * @param ron the ron
	 */
	public static void checkMappingConsistencyOfRulesWithoutAnyMarker(RON ron, IResource res){
		try{
			if (ron==null || res==null) return;
			ArrayList<Rule> rules=getAllRules(ron);
			ArrayList<String> rulesWithMarkers=new ArrayList<String>();
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);			
			for (int i=0;i<markers.length;i++){				
				rulesWithMarkers.add((String)markers[i].getAttribute("rule"));
			}
			for (Rule r : rules){
				if (!rulesWithMarkers.contains(getRuleIDString(r)))
					checkMappingConsistency(r.getLhs(), res);
			}
		} catch (NullPointerException npe){
			//ignore
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}
	
	/**
	 * @param r the rule
	 * @return ID of the rule as String
	 */
	private static String getRuleIDString(Rule r){
		if (r==null) return null;
		return Integer.toString(r.getID());
	}
	
	/**
	 * @param codom a codomain
	 * @return rule of the codomain or null if the codomain doesn't belong to a rule
	 */
	private static Rule getRule(Codomain codom){
		if (codom==null) return null;
		Rule result=null;
		if (codom instanceof RHS)
			result=((RHS)codom).getRule();
		if (codom instanceof NAC)
			result=((NAC)codom).getRule();
		return result;		
	}
	
	/**
	 * Sets a default marker from the given mcr-object,
	 * if and only if the codomain is located on a HLPlace (if it was deleted,
	 * no marker will be set). - Only works for rules!
	 * If there is no valid mapping nr (-1), the messageWithNames will be set as marker-attribute, 
	 * otherwise the messageWithMappingNrs will be set as marker-attribute,
	 * @param res resource
	 * @param mcr mapping consistency result
	 */
	private static void setMarker(IResource res, MappingConsistencyResult mcr){
		try{
			HLPlace p=null;
			Codomain cod=mcr.getCodomain();			
			if (cod instanceof LHS)
				p=((LHS)cod).getRule().getPlace();				
			else if (cod instanceof RHS)
				p=((RHS)cod).getRule().getPlace();
			else if (cod instanceof NAC)
				p=((NAC)cod).getRule().getPlace();
			else if (cod instanceof ObjectNet)
				p=((ObjectNet)cod).getPlace();			
			if (p==null) return; //token has been deleted => set no marker!
			
			IMarker marker=res.createMarker(IMarker.PROBLEM);
			if (mcr.isValidMappingNr())
				marker.setAttribute(IMarker.MESSAGE, mcr.getMessageWithMappingNrs());
			else
				marker.setAttribute(IMarker.MESSAGE, mcr.getMessageWithNames());
			marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
			marker.setAttribute(IMarker.LOCATION, createLocationStringOfMcr(mcr));
			marker.setAttribute("rule",getRuleIDString(getRule(mcr)));
			marker.setAttribute("codomain",mcr.getCodomain().getName());
		} catch (NullPointerException npe){/*ignore*/			
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}
	
	/**
	 * changes location of all markers of rule to newLocation
	 * @param res resource
	 * @param rule the rule
	 * @param newLocation new name of rule
	 */
	public static void changeMarkerLocation(IResource res, Rule rule, String newLocation){
		try{
			if (rule==null || res==null || newLocation==null) return;
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);
			for (int i=0;i<markers.length;i++){
				String identificationString=(String)markers[i].getAttribute("rule");
				if (identificationString.compareTo(getRuleIDString(rule))==0){				
					String oldString=(String)markers[i].getAttribute(IMarker.LOCATION);
					if (oldString==null) continue; //should not happen
					int index=oldString.indexOf("(");
					if (index<0) continue; //should not happen
					String newString=newLocation+" "+oldString.substring(index, oldString.length());
					markers[i].setAttribute(IMarker.LOCATION, newString);										
				}
			}
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}
	
	/**
	 * Deletes all markers with the same codomain as mcr of given resource.
	 * Only works for codomains that are RHS or NACs.
	 * @param res resource
	 * @param mcr mapping consistency result
	 */
	private static void deleteAllMarkers(IResource res, MappingConsistencyResult mcr){
		try{			
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);
			Rule r=getRule(mcr.getCodomain());
			if (r==null) return;
			for (int i=0;i<markers.length;i++){
				if (((String)markers[i].getAttribute("rule")).compareTo(getRuleIDString(r))==0 &&
					((String)markers[i].getAttribute("codomain")).compareTo(mcr.getCodomain().getName())==0){						
					markers[i].delete();
				}
			}
		} catch (NullPointerException npe){/*ignore*/
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}
	
	/**
	 * deletes all markers of given resource
	 * @param res resource
	 */
	public static void deleteAllMarkers(IResource res){
		try{
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);
			for (int i=0;i<markers.length;i++)
				markers[i].delete();			
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}

	/**
	 * deletes all markers of given rule of given resource
	 * @param res resource
	 * @param rule the ruls
	 */
	public static void deleteAllMarkersOfRule(Rule rule, IResource res){
		try{
			if (rule==null || res==null) return;
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);
			for (int i=0;i<markers.length;i++){
				if (((String)markers[i].getAttribute("rule")).compareTo(getRuleIDString(rule))==0)								
					markers[i].delete();
			}											
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}

	/**
	 * deletes all markers of non existing rules of given resource
	 * @param res resource
	 * @param ron the ron
	 */
	public static void deleteAllMarkersOfNonExistingRules(RON ron, IResource res){
		try{
			if (ron==null || res==null) return;
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);
			if (markers.length>0){
				ArrayList<String> rules=getAllRuleIDsAsStrings(ron);
				for (int i=0;i<markers.length;i++){					
					if (!rules.contains((String)markers[i].getAttribute("rule"))){
						markers[i].delete();
					}
				}				
			}			
		} catch (CoreException ce){
			throw new RuntimeException(ce);
		}
	}

	/**
	 * deletes all markers of the rules of place
	 * @param place the HLPlace
	 * @param res the resource
	 */
	public static void deleteAllMarkersOfHLPlace(HLPlace place, IResource res){
		if (place==null || res==null) return;
		if (place.getType().getValue()!=HLPlaceType.RULE) return;
		try{
			IMarker[] markers=res.findMarkers(IMarker.PROBLEM, true, 99);
			if (markers.length==0) return;
			//create names to delete
			ArrayList <String> rulesToDelete=new ArrayList<String>();
			for (Object o: place.getTokens()){
				Rule r=(Rule)o;
				rulesToDelete.add(getRuleIDString(r));			
			}
			if (rulesToDelete.isEmpty()) return;
			//delete markers
			for (int i=0;i<markers.length;i++){
				if (rulesToDelete.contains((String)markers[i].getAttribute("rule"))){
					markers[i].delete();
				}
			}			
		} catch (CoreException ce){
			throw new RuntimeException();
		}
	}
	
	
	/**
	 * returns the rule of mcr
	 * @param mcr mapping consistency result
	 * @return the rule
	 */
	private static Rule getRule(MappingConsistencyResult mcr)
	throws NullPointerException{
		Rule rule=null;
		if (mcr.getCodomain() instanceof RHS)
			rule=((RHS)mcr.getCodomain()).getRule();
		else if (mcr.getCodomain() instanceof NAC)
			rule=((NAC)mcr.getCodomain()).getRule();
		return rule;
	}
	
	/**
	 * creates the location string for a marker of the given mcr - only works for rules
	 * @param mcr mapping consistency result
	 * @return location-string for marker
	 */
	private static String createLocationStringOfMcr(MappingConsistencyResult mcr)
	throws NullPointerException{
		return getRule(mcr).getName()+" ("+mcr.getCodomain().getName()+")";
	}

	/**
	 * @return ArrayList with the IDs as Strings of all existing rules of the ron
	 */
	private static ArrayList<String> getAllRuleIDsAsStrings(RON ron){		
		ArrayList<String> l=new ArrayList<String>();
		if (ron!=null)
			for (Object o: ron.getNodes()){
				//for all nodes
				if (o instanceof HLPlace){
					//which are places
					HLPlace place=(HLPlace)o;
					if (place.getType().getValue()==HLPlaceType.RULE)
						for (Object o1: place.getTokens()){
							//for all tokens					
							l.add((getRuleIDString((Rule)o1)));					
						}
				}
			}
		return l;
	}
	
	/**
	 * @return ArrayList with all existing (HL-)tokens of the ron
	 */
	private static ArrayList<Token> getAllTokens(RON ron){
		ArrayList<Token> l=new ArrayList<Token>();
		if (ron!=null)
			for (Object o: ron.getNodes()){
				//for all nodes
				if (o instanceof HLPlace){
					//which are places
					HLPlace place=(HLPlace)o;
					for (Object o1: place.getTokens()){
						//for all tokens					
						l.add((Token)o1);					
					}
				}
			}
		return l;
	}
	
	/**
	 * @return ArrayList with all existing rules of the ron
	 */
	private static ArrayList<Rule> getAllRules(RON ron){
		ArrayList<Rule> l=new ArrayList<Rule>();
		if (ron!=null)
			for (Object o: ron.getNodes()){
				//for all nodes
				if (o instanceof HLPlace){
					//which are places
					HLPlace place=(HLPlace)o;
					if (place.getType().getValue()==HLPlaceType.RULE)
						for (Object o1: place.getTokens()){
							//for all tokens					
							l.add((Rule)o1);					
						}
				}
			}
		return l;
	}
	
}
