/**
 * 
 */
package animation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutListener;
import org.eclipse.draw2d.geometry.Rectangle;


public class AnimatorForMultiple implements LayoutListener {

	public AnimatorForMultiple() {
	}

	public void capture(final IFigure figure) {
		recordFinalState(figure);
	}

	protected void recordFinalState(final IFigure figure) {
		MultipleAnimation.putFinalState(this, figure, getCurrentState(figure));
	}

	protected void recordInitialState(final IFigure figure) {
		MultipleAnimation.putInitialState(this, figure, getCurrentState(figure));
	}

	public void init(final IFigure figure) {
		recordInitialState(figure);
	}

	/**
	 * Returns an object encapsulating the placement of children in a
	 * container. This method is called to capture both the initial and
	 * final states.
	 * 
	 * @param container
	 *            the container figure
	 * @return the current state
	 * @since 3.2
	 */
	protected Map<IFigure, Rectangle> getCurrentState(final IFigure container) {
		final Map<IFigure, Rectangle> locations = new HashMap<IFigure, Rectangle>();
		final List children = container.getChildren();
		IFigure child;
		for (int i = 0; i < children.size(); i++) {
			child = (IFigure) children.get(i);
			locations.put(child, child.getBounds().getCopy());
		}
		return locations;
	}

	/**
	 * Hooks invalidation in case animation is in progress.
	 * 
	 * @see LayoutListener#invalidate(IFigure)
	 */
	public final void invalidate(final IFigure container) {
		if (MultipleAnimation.isInitialRecording()) {
			MultipleAnimation.hookAnimator(container, this);
		}
	}

	/**
	 * Hooks layout in case animation is in progress.
	 * 
	 * @see org.eclipse.draw2d.LayoutListener#layout(org.eclipse.draw2d.IFigure)
	 */
	public final boolean layout(final IFigure container) {
		if (MultipleAnimation.isAnimating()) {
			return MultipleAnimation.hookPlayback(container, this);
		}
		return false;
	}

	/**
	 * Plays back the animated layout.
	 * 
	 * @see AnimatorForMultiple#playback(IFigure)
	 */
	protected boolean playback(final IFigure container) {
		final Map initial = (Map) MultipleAnimation
				.getInitialState(this, container);
		final Map ending = (Map) MultipleAnimation.getFinalState(this, container);
		if (initial == null) {
			return false;
		}
		final List children = container.getChildren();

		final float progress = MultipleAnimation.getProgress();
		final float ssergorp = 1 - progress;

		Rectangle rect1, rect2;

		for (int i = 0; i < children.size(); i++) {
			final IFigure child = (IFigure) children.get(i);
			rect1 = (Rectangle) initial.get(child);
			rect2 = (Rectangle) ending.get(child);

			// TODO need to change this to hide the figure until the end.
			if (rect1 == null) {
				continue;
			}
			child.setBounds(new Rectangle(Math.round(progress * rect2.x
					+ ssergorp * rect1.x), Math.round(progress * rect2.y
					+ ssergorp * rect1.y), Math.round(progress
					* rect2.width + ssergorp * rect1.width), Math
					.round(progress * rect2.height + ssergorp
							* rect1.height)));
		}
		return true;
	}

	/**
	 * Hooks post layout in case animation is in progress.
	 * 
	 * @see LayoutListener#postLayout(IFigure)
	 */
	public final void postLayout(final IFigure container) {
		if (MultipleAnimation.isFinalRecording()) {
			MultipleAnimation.hookNeedsCapture(container, this);
		}
	}

	/**
	 * This callback is unused. Reserved for possible future use.
	 * 
	 * @see LayoutListener#remove(IFigure)
	 */
	public final void remove(final IFigure child) {
	}

	/**
	 * This callback is unused. Reserved for possible future use.
	 * 
	 * @see LayoutListener#setConstraint(IFigure, Object)
	 */
	public final void setConstraint(final IFigure child,
			final Object constraint) {
	}

}