/**
 * 
 */
package roneditor.edit.parts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.edit.policies.LabelDirectEditPolicy;
import roneditor.edit.utils.LabelEditManager;
import roneditor.objectnetview.edit.parts.ObjectNetEditPart.ONNodeContainer;
import roneditor.objectnetview.model.EObjectPropertySource;
import roneditor.ronview.edit.parts.RONEditPart.HLNodeContainer;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.NamedElement;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelPackage;

public class LabelEditPart extends AbstractGraphicalEditPart implements
		Adapter {

	
	private IPropertySource propertySource = null;
	private Notifier target;

	private LabelEditManager manager;
	
	

	/**
	 * Bekommt einen HLNodeContainer (welcher ein HLPlace enthlt) 
	 * und merkt ihn sich als sein Model.
	 * @param obj Modelobjekt fr diesen EditPart
	 */
	public LabelEditPart(final Object obj) {
		setModel(obj);
	}
	
	
	/** 
	 * Erstellt die Figure (ein Label) fr diesen EditPart und setzt den Text entsprechend des Models.
	 */
	@Override
	protected IFigure createFigure() {
		Label label = new Label();
		label.setText(getCastedModel().getName());
		return label;
	}

	/**
	 * 	Bindet die Policies an eine ID;
	 * 
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE,
				new LabelDirectEditPolicy());
	}

	
	
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target= newTarget;
	}

	
	/**
	 * Holt das eigentliche Model (den passenden HLNode/ONNode) aus dem HLNodeContainer raus und gibt ihn gecasted zurck.
	 * 
	 * @return NamedElement Modelobjekt fr diesen EditPart
	 */
	public NamedElement getCastedModel() {
		Object model = this.getModel();
		
		if(model instanceof HLNodeContainer){
			return ((HLNodeContainer)model).getHLNode();
		}
		else if(model instanceof ONNodeContainer){
			return ((ONNodeContainer)model).getONNode();
		}
		else{
			return (NamedElement) model;
		}
	}
	
	
	
	/**
	 * @return Label Figure von diesem EditPart
	 */
	protected Label getLabelFigure(){
		return (Label) getFigure();
	}
	
	
	/** 
	 * Setzt den Text und die Position entsprechend des ModelObjektes neu.
	 */
	protected void refreshVisuals()
	{
		NamedElement node = getCastedModel();
		getLabelFigure().setText(node.getName());
		// Position relativ zur Place setzen
		Point loc = new Point(node.getX()+node.getLabelX(), 
							  node.getY()+node.getLabelY());
		Dimension size = new Dimension(-1,-1);
		Rectangle r = new Rectangle(loc ,size);
		((AbstractGraphicalEditPart)getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);
	}
	
		
	public Object getAdapter(Class key)
	{
		if (IPropertySource.class == key)
		{
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	protected IPropertySource getPropertySource()
	{
		if (propertySource == null)
		{
			if (getCastedModel() instanceof HLPlace) {
				propertySource = new EObjectPropertySource((HLPlace)getCastedModel());
			} else if (getCastedModel() instanceof HLTransition) {
				propertySource = new EObjectPropertySource((HLTransition)getCastedModel());
			} else if (getCastedModel() instanceof ONPlace) {
				propertySource = new EObjectPropertySource((ONPlace)getCastedModel());
			} else if (getCastedModel() instanceof ONTransition) {
				propertySource = new EObjectPropertySource((ONTransition)getCastedModel());
			}
		}
		return propertySource;
	}



	/**
	 * Falls das bergebene Request ein DirectEdit-Request ist, dann PerformDirectEdit() aufrufen. 
	 */
	public void performRequest (Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			performDirectEdit();
		}
	}

	/**
	 * Nutzt den vorhanden HLNodeEditManager oder erzeugt einen neuen falls die Funktion zum erstenmal aufgrufen wird.
	 * 
	 */
	protected void performDirectEdit() {
		if (manager == null) {
			manager = new LabelEditManager(this,
					TextCellEditor.class, ((Label)getFigure()));
		}
		manager.setName(getCastedModel().getName());
		manager.show();
	}
	
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);

		if (notification.getNotifier() instanceof NamedElement) {
			switch (featureId) {
			case RonmodelPackage.NAMED_ELEMENT__NAME:
			case RonmodelPackage.NAMED_ELEMENT__X:
			case RonmodelPackage.NAMED_ELEMENT__Y:
			case RonmodelPackage.NAMED_ELEMENT__LABEL_X:
			case RonmodelPackage.NAMED_ELEMENT__LABEL_Y:
				refreshVisuals();
				break;
			default:
			}
		}
	}
	
	@Override
	public void activate() {
		if (isActive()) {
			return;
		}

		// start listening for changes in the model
		getCastedModel().eAdapters().add(this);
		super.activate();

		refreshVisuals();
	}

	@Override
	public void deactivate() {
		if (!isActive()) {
			return;
		}

		// stop listening for changes in the model
		getCastedModel().eAdapters().remove(this);
		super.deactivate();
	}
}
