package roneditor.edit.policies;

import org.eclipse.core.resources.IResource;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.DirectEditPolicy;
import org.eclipse.gef.requests.DirectEditRequest;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.AbstractTreeEditor.EditorTreeViewer;
import roneditor.commands.RenameNamedElementCommand;
import roneditor.edit.parts.HLPlaceTreeEditPart;
import roneditor.edit.parts.HLTransitionTreeEditPart;
import roneditor.edit.parts.ObjectNetTreeEditPart;
import roneditor.edit.parts.RuleTreeEditPart;
import roneditor.objectnetview.edit.parts.ObjectNetEditPart;
import roneditor.ronview.edit.parts.HLPlaceEditPart;
import roneditor.ronview.edit.parts.HLTokenEditPart;
import roneditor.ronview.edit.parts.HLTransitionEditPart;
import ronmodel.HLNode;
import ronmodel.NamedElement;
import ronmodel.Rule;
import ronmodel.Token;
import _mapping_consistency.MappingConsistencyCheck;
import _static_help_methods.TokenName;

/**
 * An EditPolicy for EditParts of NamedElement models to handle
 * DirectEditRequests.
 * 
 * @author Tony Modica
 */
public class NamedElementDirectEditPolicy extends DirectEditPolicy {

	/**
	 * @return A RenameNamedElementCommand with the NamedElement which is the
	 *         model of the host EditPart and the new name included in the
	 *         request set.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.DirectEditPolicy#getDirectEditCommand(org.eclipse.gef.requests.DirectEditRequest)
	 */
	@Override
	protected Command getDirectEditCommand(final DirectEditRequest request) {
		NamedElement e = (NamedElement)getHost().getModel();
		String oldName=e.getName();
		final RenameNamedElementCommand command = new RenameNamedElementCommand(e);		
		String newName=(String) request.getDirectEditFeature();
		command.setNewName(newName);
		if (e instanceof Token) {
			if (!TokenName.isAllowedName(newName,((Token)e).getPlace().getRon(),false)) return null;
		} else if (e instanceof HLNode) {
			if (!TokenName.isAllowedName(newName,((HLNode)e).getRon(),false)) return null;
		} 
		
		if (e instanceof Rule){
			//change markers
			IResource res=null;
			for (Object o: e.eAdapters()){
				if (o instanceof RuleTreeEditPart){
					//important case (if no other viewer is available)
					RuleTreeEditPart rtep=(RuleTreeEditPart)o;
					res=(IResource)((EditorTreeViewer)rtep.getViewer()).getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof ObjectNetTreeEditPart){
					//important case (if no other viewer is available)
					ObjectNetTreeEditPart rtep=(ObjectNetTreeEditPart)o;
					res=(IResource)((EditorTreeViewer)rtep.getViewer()).getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof HLPlaceTreeEditPart){
					//important case (if no other viewer is available)
					HLPlaceTreeEditPart hlptep=(HLPlaceTreeEditPart)o;
					res=(IResource)((EditorTreeViewer)hlptep.getViewer()).getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof HLTransitionTreeEditPart){
					//important case (if no other viewer is available)
					HLTransitionTreeEditPart hlttep=(HLTransitionTreeEditPart)o;
					res=(IResource)((EditorTreeViewer)hlttep.getViewer()).getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof ObjectNetEditPart){
					//if objectneteditpart is available
					ObjectNetEditPart onep=(ObjectNetEditPart)o;
					res=(IResource) ((MultiViewerPageViewer)onep.getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof HLTokenEditPart){
					//if hltokeneditpart is available
					HLTokenEditPart hltep=(HLTokenEditPart)o;
					res=(IResource) ((MultiViewerPageViewer)hltep.getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof HLPlaceEditPart){
					//if hltokeneditpart is available
					HLPlaceEditPart hlpep=(HLPlaceEditPart)o;
					res=(IResource) ((MultiViewerPageViewer)hlpep.getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
					break;
				} else
				if (o instanceof HLTransitionEditPart){
					//if hltokeneditpart is available
					HLTransitionEditPart hltep=(HLTransitionEditPart)o;
					res=(IResource) ((MultiViewerPageViewer)hltep.getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
					break;
				} 
			}
			if (res==null) return null; //should NEVER happen!
			Rule rule=(Rule)e;
			MappingConsistencyCheck.changeMarkerLocation(res, rule, newName);
		}
		return command;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.DirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
	 */
	@Override
	protected void showCurrentEditValue(@SuppressWarnings("unused")
	final DirectEditRequest request) {
	}

}
