package roneditor.model;

import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelFactory;
import ronmodel.RonmodelPackage;


/**
 * This class implements the CreationFactory used by the CreationTool. It in turn
 * uses the EMF-generated factories to create the model instances
 *
 */
public class ModelCreationFactory implements CreationFactory {
	private Class targetClass;
	private HLPlaceType hlPlaceType;
	private HLTransitionKind hlTransitionKind;
	
	public ModelCreationFactory( Class targetClass ) {
		this.targetClass = targetClass;
	}
	
	public ModelCreationFactory(HLPlaceType type)
	{
		this.targetClass = HLPlace.class;
		this.hlPlaceType = type;
	}
	
	public ModelCreationFactory(HLTransitionKind kind){
		this.targetClass = HLTransition.class;
		this.hlTransitionKind = kind;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	public Object getNewObject() {
		Map registry = EPackage.Registry.INSTANCE;
		String objectNetModelURI = RonmodelPackage.eNS_URI;
		RonmodelPackage objectNetPackage =
		(RonmodelPackage) registry.get(objectNetModelURI);
		RonmodelFactory factory = objectNetPackage.getRonmodelFactory();
			
		Object result = null;
		if( targetClass.equals( ONTransition.class ) ) {
			result = factory.createONTransition();
		}
		else if (targetClass.equals(ONPlace.class)){
			result = factory.createONPlace();
		}
		else if (targetClass.equals(ONArc.class)){
			result = factory.createONArc();
		}
		else if (targetClass.equals(Mapping.class)){
			result = factory.createMapping();
		}
		else if (targetClass.equals(HLPlace.class)){
			HLPlace place = factory.createHLPlace();
			place.setType(this.hlPlaceType);
			result = place;
		}
		else if (targetClass.equals(HLTransition.class)){
			HLTransition transition = factory.createHLTransition();
			transition.setKind(this.hlTransitionKind);
			result = transition;
		}
		else if (targetClass.equals(HLArc.class)){
			result = factory.createHLArc();
		}
		
		Assert.isNotNull(result,
		"ModelCreationFactory could not create a model!");		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	public Object getObjectType() {
		return targetClass;
	}
}
