package roneditor.objectnetview;

import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import roneditor.model.ModelCreationFactory;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * Main class for the objectnet editor palette. All graphical figures have to be
 * registered here.
 */
public class ObjectNetPaletteRoot extends PaletteRoot {

	/**
	 * Creates a new ObjectNetPaletteRoot instance.
	 * 
	 */
	public ObjectNetPaletteRoot() {
		// create root
		super();

		// a group of default control tools
		PaletteGroup controls = new PaletteGroup("Controls");
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols
		PaletteGroup symbols = new PaletteGroup("Symbols");
		add(symbols);

		CreationToolEntry entry;

		entry = new ObjectNetCreationToolEntry("ONPlace", "Creates an ONPlace",
				new ModelCreationFactory(ONPlace.class), null, null);
		symbols.add(entry);

		entry = new ObjectNetCreationToolEntry("ONTransition",
				"Creates an ONTransition", new ModelCreationFactory(
						ONTransition.class), null, null);
		symbols.add(entry);

		entry = new ObjectNetConnectionCreationToolEntry("ONArc",
				"Creates an ONArc", new ModelCreationFactory(ONArc.class),
				null, null);
		symbols.add(entry);

		

		
	}
}
