package roneditor.objectnetview.actions;

/**
 * Edit action for the properties dialog,
 * where the user can change the place or transition properties.
  */
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.objectnetview.commands.EditONArcCommand;
import roneditor.objectnetview.commands.EditONPlaceCommand;
import roneditor.objectnetview.commands.EditONTransitionCommand;
import roneditor.objectnetview.dialog.ONArcPropertiesDialog;
import roneditor.objectnetview.dialog.ONPlacePropertiesDialog;
import roneditor.objectnetview.dialog.ONTransitionPropertiesDialog;
import roneditor.objectnetview.edit.parts.ONArcEditPart;
import roneditor.objectnetview.edit.parts.ONPlaceEditPart;
import roneditor.objectnetview.edit.parts.ONTransitionEditPart;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;


public class EditAction extends SelectionAction {

	static public String ID = "Properties_Dialog_Action";

	public EditAction(IWorkbenchPart part) {
		super(part);
		setDescription("Properties Dialog Action");
		setText("Properties Dialog");
	}

	
	/* (non-Javadoc)
	* @see org.eclipse.jface.action.IAction#run()
	*/
	public void run() {
		execute(getCommand());
	}
	
	/*
	 * Get the command class from the place or transition edit part.
	 */
	public Command getCommand(){
		Object selection =
			((StructuredSelection) getSelection()).getFirstElement();
			
		if (selection instanceof ONTransitionEditPart){
			ONTransition ot = ((ONTransitionEditPart)selection).getONTransition();
			EditONTransitionCommand editCommand = new EditONTransitionCommand();
			editCommand.setONTransition(ot);			
			ONTransitionPropertiesDialog d = new ONTransitionPropertiesDialog(new Shell(), SWT.NULL);
			d.setText("Edit ONTransition " + ot.getName());
			d.setName(ot.getName());
			d.open();
			if(d.isCancelled()){
				return null;
			}
			editCommand.setNewName(d.getName());
			return editCommand; 
		} else if (selection instanceof ONPlaceEditPart){
			ONPlace op = ((ONPlaceEditPart)selection).getONPlace();
			EditONPlaceCommand editCommand = new EditONPlaceCommand();
			editCommand.setONPlace(op);			
			ONPlacePropertiesDialog d = new ONPlacePropertiesDialog(new Shell(), SWT.NULL);
			d.setText("Edit ONPlace " + op.getName());
			d.setName(op.getName());
			d.setTokens(op.getTokens());
			d.open();
			if(d.isCancelled()){
				return null;
			}			
			editCommand.setNewName(d.getName());
			editCommand.setNewTokens(d.getTokens());
			return editCommand; 
		} else if (selection instanceof ONArcEditPart){
			ONArc oa = ((ONArcEditPart)selection).getONArc();
			EditONArcCommand editCommand = new EditONArcCommand();
			editCommand.setONArc(oa);			
			ONArcPropertiesDialog d = new ONArcPropertiesDialog(new Shell(), SWT.NULL);
			d.setText("Edit ONArc");
			d.setWeight(oa.getWeight());
			d.open();
			if(d.isCancelled()){
				return null;
			}
			editCommand.setNewWeight(d.getWeight());
			return editCommand; 
		}
		return null;
		
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
			return canPerformAction();
	}
	
	/*
	 * Returns true if the action could be executed, false otherwise.
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		if (parts.size()==1){
			Object o = parts.get(0);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;			
			if (part.getModel() instanceof ONNode || part.getModel() instanceof ONArc)
				return true;
			else
				return false;
		}
		return false;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
