package roneditor.objectnetview.commands;

import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

public abstract class AbstractConnectionCommand extends Command {

	private static final String ConnectionCommand_Label = "ONArc";
	
	protected ONArc onarc;
	protected ONNode source;
	protected ONNode target;
	protected int weight;
	
	protected int anchorNrT;
	protected int anchorNrS;
	
		
	/**
	 * Constructor.
	 */
	public AbstractConnectionCommand() {
		super(ConnectionCommand_Label);
	}

	public abstract boolean canExecute();
	
	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		onarc.setWeight(weight);
		onarc.setAnchorNrS(anchorNrS);
		onarc.setAnchorNrT(anchorNrT); 
		ObjectNet on = target.getObjectNet();
		onarc.setObjectNet(on);
		onarc.setSource(source);
		onarc.setTarget(target);
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		onarc.setObjectNet(null);
		onarc.setSource(null);
		onarc.setTarget(null);
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	
	/**
	 * @return Returns the anchor number of the source.
	 */
	public int getAnchorNrS() {
		return anchorNrS;
	}// getAnchorNrS

	/**
	 * @return Returns the anchor number of the target.
	 */
	public int getAnchorNrT() {
		return anchorNrT;
	}// getAnchorNrT

	/**
	 * @param anchorNrS
	 *            The anchor number of the source to set.
	 */
	public void setAnchorNrS(int anchorNrS) {
		this.anchorNrS = anchorNrS;
	}// setAnchorNrS

	/**
	 * @param anchorNrT
	 *            The anchor number of the source to set.
	 */
	public void setAnchorNrT(int anchorNrT) {
		this.anchorNrT = anchorNrT;
	}// setAnchorNrT

	public void setONArc(ONArc onarc) {
		this.onarc = onarc;
	}

	/**
	 * @return the onarc
	 */
	public ONArc getONArc() {
		return onarc;
	}

	protected void setSource(ONNode source) {
		this.source = source;
	}

	protected void setTarget(ONNode target) {
		this.target = target;
	}

	/**
	 * @return the source
	 */
	public ONNode getSource() {
		return source;
	}

	/**
	 * @return the target
	 */
	public ONNode getTarget() {
		return target;
	}

	/**
	 * @return the weight
	 */
	public int getWeight() {
		return weight;
	}

	/**
	 * @param weight the name to set
	 */
	public void setWeight(int weight) {
		this.weight = weight;
	}

}
