package roneditor.objectnetview.commands;

import java.util.Vector;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * Command for deleting a transition.
 */
public class DeleteONNodeCommand 
		extends org.eclipse.gef.commands.Command {
	private static final String DeleteCommand_Label = "DeleteONNodeCommand";
	private	 ONNode onNode;
	private ObjectNet objectNet;
	private Vector<DeleteONArcCommand> deleteONArcCommands= new Vector<DeleteONArcCommand>();

	/**
	 * 
	 */
	public DeleteONNodeCommand() {
		super( DeleteCommand_Label );
	}

	/**
	 * Create delete-commands for all arcs which are connected with the node.
	 * @param onNode
	 */
	private void createDeleteONArcCommands(ONNode onNode) {
		deleteONArcCommands.clear();
		for (Object o: onNode.getOutgoing()){			
			DeleteONArcCommand cmd=new DeleteONArcCommand();
			cmd.setONArc((ONArc)o);
			deleteONArcCommands.add(cmd);
		}
		for (Object o: onNode.getIncoming()){			
			DeleteONArcCommand cmd=new DeleteONArcCommand();
			cmd.setONArc((ONArc)o);
			deleteONArcCommands.add(cmd);
		}
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {
		createDeleteONArcCommands(onNode);
		for(DeleteONArcCommand cmd: deleteONArcCommands)
			cmd.execute();
		onNode.setObjectNet(null);
	}

	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		onNode.setObjectNet(objectNet);		
		for(DeleteONArcCommand cmd: deleteONArcCommands)
			cmd.undo();
	}

	/**
	 * @return the objectNet
	 */
	public ObjectNet getObjectNet() {
		return objectNet;
	}

	/**
	 * @param objectNet the objectNet to set
	 */
	public void setObjectNet(ObjectNet objectNet) {
		this.objectNet = objectNet;
	}

	/**
	 * @return the onNode
	 */
	public ONNode getONNode() {
		return onNode;
	}

	/**
	 * @param onNode the onNode to set
	 */
	public void setONNode(ONNode onNode) {
		this.onNode = onNode;
	}

}
