package roneditor.objectnetview.edit.parts;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.resources.IResource;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.RelativeBendpoint;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.objectnetview.edit.policies.ArcSelectionHandlesEditPolicy;
import roneditor.objectnetview.edit.policies.ConnectionBendPointEditPolicy;
import roneditor.objectnetview.edit.policies.ONArcEditPolicy;
import roneditor.objectnetview.edit.policies.ONArcEndpointEditPolicy;
import ronmodel.BendPoint;
import ronmodel.ONArc;
import ronmodel.RonmodelPackage;
import _mapping_consistency.MappingConsistencyCheck;

/**
 * EditPart for a onArc.
 */
public class ONArcEditPart extends AbstractConnectionEditPart {

	private Notifier target;

	private OnArcAdapter adapter;

	private final Object childObject = new Object();

	public ONArcEditPart(ONArc onArc) {
		super();
		setModel(onArc);
	}

	public ONArc getONArc() {
		return (ONArc) getModel();
	}

	/**
	 * @return A List that contains simply an Object as pseudo model for the
	 *         ONArcLabelEditPart second be created
	 */
	@Override
	protected List getModelChildren() {
		final List childrenList = new Vector();
		childrenList.add(childObject);
		return childrenList;
	}
	
	
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ONArcEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new ConnectionBendPointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new ONArcEditPolicy());
	}

	@Override
	protected IFigure createFigure() {
		PolylineConnection c = (PolylineConnection) super.createFigure();
		// target decoration -->
		// arrow at the end of the line
		PolygonDecoration decoration = new PolygonDecoration();
		c.setTargetDecoration(decoration);

		ConnectionEndpointLocator relationshipLocator = new ConnectionEndpointLocator(
				c, false);
		relationshipLocator.setUDistance(5);
		relationshipLocator.setVDistance(10);
		c.setConnectionRouter(new BendpointConnectionRouter());
		return c;
	}

	@Override
	public void activate() {
		adapter = new OnArcAdapter();
		getONArc().eAdapters().add(adapter);
		super.activate();
	}

	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		Connection c = getConnectionFigure();

		ConnectionEndpointLocator relationshipLocator = new ConnectionEndpointLocator(
				c, false);
		relationshipLocator.setUDistance(5);
		relationshipLocator.setVDistance(10);

		refreshBendpoints();
	}

	protected void refreshBendpoints() {
		List figureConstraint = new ArrayList();
		for (Object modelConstraint : getONArc().getBendpoints()) {
			BendPoint wbp = (BendPoint) modelConstraint;
			// RelativeBendpoint rbp = new
			// RelativeBendpoint(getConnectionFigure());
			RelativeBendpoint rbp = new RelativeBendpoint(
					(Connection) getConnectionFigure());

			Dimension first = new Dimension((Integer) wbp.getW().get(0),
					(Integer) wbp.getH().get(0));
			Dimension second = new Dimension((Integer) wbp.getW().get(1),
					(Integer) wbp.getH().get(1));
			rbp.setRelativeDimensions(first, second);
			figureConstraint.add(rbp);
		}
		getConnectionFigure().setRoutingConstraint(figureConstraint);
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class OnArcAdapter implements Adapter {
		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.ON_ARC__ANCHOR_NR_S:
				refreshSourceAnchor();
				break;
			case RonmodelPackage.ON_ARC__ANCHOR_NR_T:
				refreshTargetAnchor();
				break;
			case RonmodelPackage.ON_ARC__BENDPOINTS:
				refreshBendpoints();
				break;
			case RonmodelPackage.ON_ARC__SOURCE:
			case RonmodelPackage.ON_ARC__TARGET:
			case RonmodelPackage.ON_ARC__WEIGHT:
				refreshVisuals();
				try{
					if (getViewer() instanceof MultiViewerPageViewer) {
						IResource res=(IResource) ((MultiViewerPageViewer)getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
						MappingConsistencyCheck.checkMappingConsistency(((ONArc)getModel()).getObjectNet(),res);
					}
				} catch (NullPointerException npe){/*BH*/}
				break;
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(getModel().getClass());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class
}
