package roneditor.objectnetview.edit.parts;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.edit.parts.LabelEditPart;
import roneditor.objectnetview.figures.AbstractONNodeFigure;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;
import _mapping_consistency.MappingConsistencyCheck;

public abstract class ONNodeEditPart	
	extends AbstractGraphicalEditPart
	implements NodeEditPart, Adapter
{


	private LabelEditPart labelEditPart;		
	protected List<Mapping> oldSourceMappings;
	
	
		
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {		
		int featureId = notification.getFeatureID( RonmodelPackage.class );
		switch( featureId ) {
			//case -1:
		case RonmodelPackage.ON_NODE__OUTGOING_M: 		

		case RonmodelPackage.ON_NODE__INCOMING_M:
		case RonmodelPackage.ON_NODE__OBJECT_NET:
		case RonmodelPackage.ON_NODE__OUTGOING:				
		case RonmodelPackage.ON_NODE__INCOMING:
			
		consistencyCheck((ObjectNet)getParent().getModel());
		}		
	}
	
	/**
	 * Does a consistency check for given objectnet (see MappingConsistencyCheck.java)
	 * @param on ObjectNet
	 */
	protected void consistencyCheck(ObjectNet on){
		if (getViewer() instanceof MultiViewerPageViewer) {
			IResource res=(IResource) ((MultiViewerPageViewer)getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
			MappingConsistencyCheck.checkMappingConsistency(on,res);
		}
	}
	

	/**
	 * 
	 * @return zugehrigen LabelEditPart
	 */
	public LabelEditPart getLabelEditPart() {
		if (labelEditPart == null) {
			Iterator iter = getParent().getChildren().iterator();
			EditPart editPart = null;
			while (iter.hasNext()) {
				if (((editPart = (EditPart)iter.next()) instanceof LabelEditPart) 
						&& (((LabelEditPart)editPart).getCastedModel() == this.getModel())) {
					labelEditPart = (LabelEditPart)editPart;
					break;
				}
			}
		}
		return labelEditPart;
	}
	

	
	protected Rule getRule() {
		Rule rule = null;
		if (getParent().getModel() instanceof LHS) {
			rule = ((LHS)getParent().getModel()).getRule();
		} else if (getParent().getModel() instanceof RHS) {
			rule = ((RHS)getParent().getModel()).getRule();
		} else if (getParent().getModel() instanceof NAC) {
			rule = ((NAC)getParent().getModel()).getRule();
		}
		return rule;
	}
	
	private AbstractONNodeFigure getONNodeFigure() {
		return (AbstractONNodeFigure)getFigure();
	}
	
	private ONNode getONNode() {
		return (ONNode)getModel();
	}
	
}
