package roneditor.objectnetview.edit.parts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.actions.AddTokenAction;
import roneditor.objectnetview.edit.policies.ONNodeComponentEditPolicy;
import roneditor.objectnetview.edit.policies.ONPlaceEditPolicy;
import roneditor.objectnetview.figures.ONPlaceFigure;
import roneditor.objectnetview.model.EObjectPropertySource;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a ONPlace.
 */
public class ONPlaceEditPart 
	extends ONNodeEditPart
	implements NodeEditPart, Adapter
{
	private IPropertySource propertySource = null;
	private Notifier target;
	
	/**
	 * Creates a new ONPlaceEditPart instance.
	 * @param element
	 */
	public ONPlaceEditPart(ONPlace onPlace)
	{
		setModel(onPlace);
		if (getONPlace().getObjectNet() instanceof LHS) {
			oldSourceMappings= new ArrayList<Mapping>();
			oldSourceMappings.addAll(getONPlace().getOutgoingM());
		}
	}
	

	

	public ONPlace getONPlace() {
		return (ONPlace)getModel();		
	}
	
	


	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getONPlace().getOutgoing();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getONPlace().getIncoming();
	}
	

	/** 
	 * Returns the Figure of this, as a ONPlace type figure.
	 *
	 * @return  Figure as a ONPlaceFigure.
	 */
	protected ONPlaceFigure getONPlaceFigure(){
		return (ONPlaceFigure) getFigure();
	}
	
	public void notifyChanged(Notification notification) {
		super.notifyChanged(notification);
		int featureId = notification.getFeatureID( RonmodelPackage.class );
		switch( featureId ) {
			case RonmodelPackage.ON_PLACE__OUTGOING:				
				refreshSourceConnections();
				//refreshVisuals of all post-transitions (checkEnabled)
				for (Object o: getONPlace().getOutgoing()){
					try{
						ONTransition t=(ONTransition)((ONArc)o).getTarget();
						for (Object o1: t.eAdapters()){
							if (o1 instanceof ONTransitionEditPart)
								((ONTransitionEditPart)o1).refreshVisuals();
						}
					} catch(NullPointerException npe){/*ignore*/}
				}
				break;		
			case RonmodelPackage.ON_PLACE__INCOMING:
				refreshTargetConnections();				
				break;
			case RonmodelPackage.ON_PLACE__TOKENS:
				for (Object o : this.getSourceConnections())
				{
					((ConnectionEditPart)o).getTarget().refresh();
				}
			case RonmodelPackage.ON_PLACE__OUTGOING_M: 
			case RonmodelPackage.ON_PLACE__INCOMING_M: 
			case RonmodelPackage.ON_PLACE__NAME:
			case RonmodelPackage.ON_PLACE__X:
			case RonmodelPackage.ON_PLACE__Y:	
				this.refreshVisuals();
				break;
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, new ONPlaceEditPolicy() );
		// install the edit policy to handle deletion
		installEditPolicy(
			EditPolicy.COMPONENT_ROLE,
			new ONNodeComponentEditPolicy());
	}
	
	/**
	 * refreshing the mouse point according to the request
	 * 
	 * @param point
	 */
	protected Point calculatePoint(Request req) {
		Point p = new Point(0, 0);
		if (req instanceof CreateConnectionRequest) {
			CreateConnectionRequest r = (CreateConnectionRequest) req;
			p = r.getLocation();
		} else if (req instanceof ReconnectRequest) {
			ReconnectRequest r2 = (ReconnectRequest) req;
			p = r2.getLocation();
		}
		return p;
	}// calculatePoint
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		ConnectionAnchor sourceAnchor;
		
		int anchorNr =  ((ONArc) connection.getModel()).getAnchorNrS();
		if (anchorNr!=-1)
			sourceAnchor = getONPlaceFigure().getSourceConnectionAnchors().get(anchorNr);
		else
			sourceAnchor= getONPlaceFigure().getSourceCBConnectionAnchor();
		return sourceAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {		
		//return getONPlaceFigure().getSourceConnectionAnchor();
		Point mousePoint = calculatePoint(request);		
		return getONPlaceFigure().getSourceConnectionAnchorAt(mousePoint); 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {		
		ConnectionAnchor targetAnchor;

		int anchorNr = ((ONArc) connection.getModel()).getAnchorNrT();			
		if (anchorNr!=-1)
			targetAnchor = getONPlaceFigure().getTargetConnectionAnchors().get(anchorNr);
		else
			targetAnchor = getONPlaceFigure().getTargetCBConnectionAnchor();
		return targetAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {		
		Point mousePoint = calculatePoint(request);		
		return getONPlaceFigure().getTargetConnectionAnchorAt(mousePoint); 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		// Mapping Nummer setzen
		List<Mapping> mappings;
		if ((mappings = getONPlace().getOutgoingM()).size() > 0) {
			getONPlaceFigure().setMappingNr(mappings.get(0).getMappingNr());
		} else if ((mappings = getONPlace().getIncomingM()).size() > 0) {
			getONPlaceFigure().setMappingNr(mappings.get(0).getMappingNr());
		} else {
			getONPlaceFigure().setMappingNr(-1);
		}
		getONPlaceFigure().setTokens( getONPlace().getTokens() );
		Point loc = new Point( getONPlace().getX(), getONPlace().getY() );
		Dimension size = new Dimension(-1,-1);
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);
	}



	
	
	
	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		((Notifier)getONPlace()).eAdapters().add(this);

		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier)getONPlace()).eAdapters().remove(this);

		super.deactivate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getONPlace() );
		}
		return propertySource;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		ONPlaceFigure figure = new ONPlaceFigure();

		return figure;
	}

	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		if (req.getType().equals(RequestConstants.REQ_OPEN)) {

			if (!(getViewer() instanceof MultiViewerPageViewer)) { return; }
			//getParent().performRequest(req);
			(((MultiViewerPageViewer)getViewer()).getHostPage()).getActionRegistry().getAction(AddTokenAction.ID).run();
		}
	}
	
	
}
