package roneditor.objectnetview.edit.parts;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.actions.FireTransitionAction;
import roneditor.objectnetview.edit.policies.ONNodeComponentEditPolicy;
import roneditor.objectnetview.edit.policies.ONTransitionEditPolicy;
import roneditor.objectnetview.figures.ONTransitionFigure;
import roneditor.objectnetview.model.EObjectPropertySource;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelPackage;


/**
 * EditPart for a ONTransition. 
 */
public class ONTransitionEditPart 
	extends ONNodeEditPart
	implements NodeEditPart, Adapter
{
	private IPropertySource propertySource = null;
	private Notifier target;
	
	
	/**
	 * Creates a new ONTransitionEditPart instance.
	 * @param element
	 */
	public ONTransitionEditPart(ONTransition onTransition)
	{
		setModel(onTransition);
		if (getONTransition().getObjectNet() instanceof LHS) {
			oldSourceMappings= new ArrayList<Mapping>();
			oldSourceMappings.addAll(getONTransition().getOutgoingM());
		}
	}

	public ONTransition getONTransition() {
		return (ONTransition)getModel();
	}

	

	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getONTransition().getOutgoing();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getONTransition().getIncoming();
	}
	

	/** 
	 * Returns the Figure of this, as a ONTransition type figure.
	 *
	 * @return  Figure as a ONTransitionFigure.
	 */
	protected ONTransitionFigure getONTransitionFigure(){
		return (ONTransitionFigure) getFigure();
	}



	public void notifyChanged(Notification notification) {
		super.notifyChanged(notification);
		int featureId = notification.getFeatureID(RonmodelPackage.class );		
		switch( featureId ){ 
			case RonmodelPackage.ON_TRANSITION__OUTGOING:				
				refreshSourceConnections();	
				refreshVisuals();			
				break;		
			case RonmodelPackage.ON_TRANSITION__INCOMING:				
				refreshTargetConnections();					
				refreshVisuals();			
				break;
			case RonmodelPackage.ON_TRANSITION__OUTGOING_M: 				
				case RonmodelPackage.ON_TRANSITION__INCOMING_M: 
				case RonmodelPackage.ON_TRANSITION__NAME:
				case RonmodelPackage.ON_TRANSITION__X:
				case RonmodelPackage.ON_TRANSITION__Y:	
					this.refreshVisuals();	
					break;
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, new ONTransitionEditPolicy() );
		// install the edit policy to handle deletion
		installEditPolicy(
			EditPolicy.COMPONENT_ROLE,
			new ONNodeComponentEditPolicy());
	}
	
	/**
	 * refreshing the mouse point according to the request
	 * 
	 * @param point
	 */
	protected Point calculatePoint(Request req) {
		Point p = new Point(0, 0);
		if (req instanceof CreateConnectionRequest) {
			CreateConnectionRequest r = (CreateConnectionRequest) req;
			p = r.getLocation();
		} else if (req instanceof ReconnectRequest) {
			ReconnectRequest r2 = (ReconnectRequest) req;
			p = r2.getLocation();
		}
		return p;
	}// calculatePoint
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		ConnectionAnchor sourceAnchor;
		int anchorNr = ((ONArc) connection.getModel()).getAnchorNrS();
		if (anchorNr!=-1)
			sourceAnchor = getONTransitionFigure().getSourceConnectionAnchors().get(anchorNr);
		else
			sourceAnchor = getONTransitionFigure().getSourceCBConnectionAnchor();
		return sourceAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {		
		Point mousePoint = calculatePoint(request);		
		return getONTransitionFigure().getSourceConnectionAnchorAt(mousePoint);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {		
		ConnectionAnchor targetAnchor;

		int anchorNr = ((ONArc) connection.getModel()).getAnchorNrT();			
		if (anchorNr!=-1)
			targetAnchor = getONTransitionFigure().getTargetConnectionAnchors().get(anchorNr);
		else
			targetAnchor = getONTransitionFigure().getTargetCBConnectionAnchor();
		return targetAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override	
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {		
		//return getONTransitionFigure().getTargetConnectionAnchor( );
		Point mousePoint = calculatePoint(request);		
		return getONTransitionFigure().getTargetConnectionAnchorAt(mousePoint); 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		// Mapping Nummer setzen
		List<Mapping> mappings;
		if ((mappings = getONTransition().getOutgoingM()).size() > 0) {
			getONTransitionFigure().setMappingNr(mappings.get(0).getMappingNr());
		} else if ((mappings = getONTransition().getIncomingM()).size() > 0) {
			getONTransitionFigure().setMappingNr(mappings.get(0).getMappingNr());
		} else {
			getONTransitionFigure().setMappingNr(-1);
		}
		Point loc = new Point( getONTransition().getX(), getONTransition().getY() );
		Dimension size = new Dimension(-1,-1);
		
		if(fireEnabled()){
			//getONTransitionFigure().setBackgroundColor(ColorConstants.lightGreen);
			getONTransitionFigure().setColor(ColorConstants.lightGreen);
		}
		else{
			//getONTransitionFigure().setBackgroundColor(ColorConstants.red);
			getONTransitionFigure().setColor(ColorConstants.red);
		}
		
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);
	}
	

	
	public boolean fireEnabled(){
		
		List incoming = getONTransition().getIncoming();
		ListIterator li = incoming.listIterator();
		
		while (li.hasNext()){
			ONArc oa = (ONArc)li.next();
			if (oa == null) return false;	
			
			ONPlace op = (ONPlace)oa.getSource();			
			if (op == null) return false;	
			if(oa.getWeight()>op.getTokens()) return false;
		}
		return true;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		
		((Notifier)getONTransition()).eAdapters().add(this);

		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier)getONTransition()).eAdapters().remove(this);

		super.deactivate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getONTransition() );
		}
		return propertySource;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		ONTransitionFigure figure = new ONTransitionFigure();

		return figure;
	}
	
	
	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		if (req.getType().equals(RequestConstants.REQ_OPEN)) {
			//getParent().performRequest(req);
			if (!(getViewer() instanceof MultiViewerPageViewer)) { return; }
			FireTransitionAction fireAction = (FireTransitionAction)(((MultiViewerPageViewer)getViewer()).getHostPage()).getActionRegistry().getAction(FireTransitionAction.ID);
			if (fireAction.isEnabled()) {
				fireAction.run();
			}
		}
	}
	
	
	
}
