package roneditor.objectnetview.edit.parts;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.objectnetview.edit.policies.ObjectNetEditPolicy;
import roneditor.objectnetview.edit.policies.ObjectNetXYLayoutEditPolicy;
import roneditor.objectnetview.model.EObjectPropertySource;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;

/**
 * EditPart for the main component "ObjectNet".
 */
public class ObjectNetEditPart extends AbstractGraphicalEditPart implements Adapter  {
	private IPropertySource propertySource = null;
	private Notifier target;
	private Shell shell;
	private Label label;

//	public static List<Mapping> allMappings= new ArrayList<Mapping>();
	
	// Hilfsklasse
	public class ONNodeContainer {
		private ONNode onNode;
		
		public ONNodeContainer(ONNode onNode) {
			this.onNode = onNode;
		}
		
		public ONNode getONNode() {
			return onNode;
		}
	}
	
	/**
	 * @param element
	 */
	public ObjectNetEditPart(ObjectNet objectNet) {
		label = null;
		setModel(objectNet);
	}



	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure()
	{
		FreeformLayer layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());
		layer.setBorder(new LineBorder(1));
		return layer;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies()
	{
	
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new ObjectNetEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new ObjectNetXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.NODE_ROLE, null);
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, null);
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());    	
	}

		
	
	/**
	 * Returns the objectNet.
	 * @return the objectNet
	 */
	public ObjectNet getObjectNet()
	{
		return (ObjectNet) getModel();
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@SuppressWarnings("unchecked")
	protected List getModelChildren()
	{
		List children = new Vector();
		// alle ONNodes einfgen
		children.addAll(getObjectNet().getNodes());
		Iterator iter = getObjectNet().getNodes().iterator();
		while(iter.hasNext()) {
			ONNode onNode = (ONNode)(iter.next());
			// Einen ONNodeContainer zu den Modelchilds hinzufgen
			// dieser Umweg ist ntig, da 2 Editparts die am gleichen Modelelement hngen,
			// Probleme beim Lschen verursachen.
			// Es hat nun also jedes PlaceLabel als Modelelement einen ONNodeContainer, 
			// dass es die Klasse im eigentlichen Model garnicht gibt, strt erstmal nicht. 
			children.add(new ONNodeContainer(onNode));
		}		
		return children;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		((Notifier)getObjectNet()).eAdapters().add(this);
		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier)getObjectNet()).eAdapters().remove(this);

		super.deactivate();
	}



	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/* override the default behavior defined in AbstractEditPart
		*  which would expect the model to be a property sourced. 
		*  instead the editpart can provide a property source
		*/
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getObjectNet() );
		}
		return propertySource;
		
	}
	
	/**
	 * Notify changes from the model to the view.
	 */
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		switch( type ) {
			case Notification.ADD:
			case Notification.ADD_MANY:
			case Notification.REMOVE:
			case Notification.REMOVE_MANY:
				refreshChildren();
				break;
			case Notification.SET:
				refreshVisuals();
				break;				
		}
		int featureId = notification.getFeatureID( RonmodelPackage.class );
		switch( featureId ) {
			case RonmodelPackage.OBJECT_NET__NAME:
				refreshVisuals();
				break;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		if (label == null) {
			label = new Label(getObjectNet().getName());
			final FontData data = new FontData();
			data.setStyle(SWT.ITALIC);
			data.setHeight(20);
			Font f = new Font(null, data);
			label.setFont(f);
			label.setOpaque(true);
			this.getFigure().add(label);
		}
		label.setText(getObjectNet().getName());
		Rectangle r = new Rectangle(10 , 10 , -1 , -1);
		this.setLayoutConstraint(this,label,r);
	}
	
	
	
	/**
	 * @return Returns the shell.
	 */
	public Shell getShell() {
		return shell;
	}
	/**
	 * @param shell The shell to set.
	 */
	public void setShell(Shell shell) {
		this.shell = shell;
	}

}
