package roneditor.objectnetview.edit.policies;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.objectnetview.anchors.ONAnchor;
import roneditor.objectnetview.commands.ConnectionONPlaceONTransCommand;
import roneditor.objectnetview.commands.ConnectionONTransONPlaceCommand;
import roneditor.objectnetview.commands.DeleteMappingCommand;
import roneditor.objectnetview.commands.MappingONPlaceCommand;
import roneditor.objectnetview.commands.ReconnectONArcCommand;
import roneditor.objectnetview.edit.parts.ONPlaceEditPart;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;


/**
 * EditPolicy for a ONPlace.
 */
public class ONPlaceEditPolicy 
	extends ONNodeEditPolicy {


	public ONPlaceEditPolicy() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();		
		if (command instanceof ConnectionONTransONPlaceCommand) {	
			// ONArc ConnectionComplete
			ConnectionONTransONPlaceCommand commandTP = (ConnectionONTransONPlaceCommand)command;	
			if (commandTP.getSource() == getONPlace()) return null;
			commandTP.setTarget(getONPlace());
			if (thereIsAlreadyONArc(commandTP)) return null;
			ConnectionAnchor anchor = ((ONPlaceEditPart) getHost())
			.getTargetConnectionAnchor(request);
			if (anchor instanceof ONAnchor)
				commandTP.setAnchorNrT(((ONAnchor)anchor).getId());
			else
				commandTP.setAnchorNrT(-1);
			//request.setStartCommand(command);

		} else if (command instanceof MappingONPlaceCommand) {
			MappingONPlaceCommand commandP = (MappingONPlaceCommand)command;
			if (commandP.getSource() == null) return null;
			if (commandP.getSource() == getONPlace()) return null;
			// Mapping ConnectionComplete
			// Mappings drfen nicht in der LHS enden
			if (getONPlace().getObjectNet() instanceof LHS) return null;
			// dieses ONPlace als Target setzen
			commandP.setTarget(getONPlace());
			// suchen nach strenden vorhanden Mappings
			Mapping tempMapping= null;
			Mapping tempMapping2= null;
			// 1. Das zu erstellende Mapping schon vorhanden, dann lschen des Mappings.
			if ((tempMapping=thereIsAlreadyMapping(commandP)) != null) {
				DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
				deleteCommand.setMapping(tempMapping);
				return deleteCommand;
			// 2. Ein anderes Mapping zwischen Source und einer andren ONPlace auf der Seite(RHS,NAC1,NAC2,...) dieser ONPlace vorhanden.
			// Dann altes lschen und neues einfgen
			} else if (((tempMapping= thereIsAlreadyMappingToSide(commandP)) != null)
					| ((tempMapping2= thereIsAlreadyMappingFromLHS(commandP)) != null)) {		
				CompoundCommand cmd= new CompoundCommand();
				if (tempMapping != null) {
					DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
					deleteCommand.setMapping(tempMapping);
					cmd.add(deleteCommand);
				}
				if (tempMapping2 != null) {
					DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
					deleteCommand.setMapping(tempMapping2);
					cmd.add(deleteCommand);
				}
				cmd.add(commandP);
				return cmd;
			}			
		} else return null; //not possible
		return command;
	}
	
	protected ONPlace getONPlace() {
		return (ONPlace)getHost().getModel();
	}
	
	protected ONPlaceEditPart getONPlaceEditPart() {
		return (ONPlaceEditPart)getHost();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {		
		Object reqObj = request.getNewObject();
		Command command = null;
		if (reqObj instanceof ONArc) {
			ConnectionONPlaceONTransCommand cmd = new ConnectionONPlaceONTransCommand();
			cmd.setONArc((ONArc)reqObj);
			cmd.setSource(getONPlace());
			ConnectionAnchor anchor = ((ONPlaceEditPart) getHost())
			.getSourceConnectionAnchor(request);
			if (anchor instanceof ONAnchor)
				cmd.setAnchorNrS(((ONAnchor)anchor).getId());
			else
				cmd.setAnchorNrS(-1);
			command= cmd;
		} else if (reqObj instanceof Mapping) {
			if (!(getONPlace().getObjectNet() instanceof LHS)) return null;
			MappingONPlaceCommand cmd = new MappingONPlaceCommand();
			cmd.setMapping((Mapping)reqObj);
			cmd.setSource(getONPlace());
			command= cmd;
		}			
		request.setStartCommand(command);
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
		protected Command getReconnectTargetCommand(ReconnectRequest request) {
		ONArc onArc = (ONArc) request.getConnectionEditPart().getModel();
		
		if (onArc.getTarget() instanceof ONTransition) {			
			return null;
		}			
		ReconnectONArcCommand command = new ReconnectONArcCommand(onArc);
		ConnectionAnchor anchor = ((ONPlaceEditPart) getHost()).getTargetConnectionAnchor(request);
		if (anchor instanceof ONAnchor)
			command.setNewAnchorNrT(((ONAnchor)anchor).getId());
		else
			command.setNewAnchorNrT(-1);
		command.setNewTarget(getONPlace());
		if (thereIsAlreadyONArc(command)) return null;
		if (nothingChanged(command)) return null;
		return command;
	}

		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
		 */
		protected Command getReconnectSourceCommand(ReconnectRequest request) {
			ONArc onArc = (ONArc) request.getConnectionEditPart().getModel();
			
			if (onArc.getSource() instanceof ONTransition) {			
				return null;
			}				
			ReconnectONArcCommand command = new ReconnectONArcCommand(onArc);
			ConnectionAnchor anchor = ((ONPlaceEditPart) getHost())
					.getSourceConnectionAnchor(request);
			if (anchor instanceof ONAnchor)
				command.setNewAnchorNrS(((ONAnchor)anchor).getId());
			else
				command.setNewAnchorNrS(-1);
			command.setNewSource(getONPlace());
			if (thereIsAlreadyONArc(command)) return null;
			if (nothingChanged(command)) return null;
			return command;
	}
}
