package roneditor.objectnetview.edit.policies;


import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.objectnetview.anchors.ONAnchor;
import roneditor.objectnetview.commands.ConnectionONPlaceONTransCommand;
import roneditor.objectnetview.commands.ConnectionONTransONPlaceCommand;
import roneditor.objectnetview.commands.DeleteMappingCommand;
import roneditor.objectnetview.commands.MappingONTransitionCommand;
import roneditor.objectnetview.commands.ReconnectONArcCommand;
import roneditor.objectnetview.edit.parts.ONTransitionEditPart;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;


/**
 * EditPolicy for a ONTransition.
 */
public class ONTransitionEditPolicy 
	extends ONNodeEditPolicy {


	public ONTransitionEditPolicy() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		if (command instanceof ConnectionONPlaceONTransCommand) {
			// ONArc ConnectionComplete
			ConnectionONPlaceONTransCommand commandPT = (ConnectionONPlaceONTransCommand)command;	
			if (commandPT.getSource() == getONTransition()) return null;		
			commandPT.setTarget(getONTransition());
			if (thereIsAlreadyONArc(commandPT)) return null;
			ConnectionAnchor anchor = ((ONTransitionEditPart) getHost())
			.getTargetConnectionAnchor(request);
			if (anchor instanceof ONAnchor)
				commandPT.setAnchorNrT(((ONAnchor)anchor).getId());
			else
				commandPT.setAnchorNrT(-1);		
		}  else if (command instanceof MappingONTransitionCommand) {
			// Mapping ConnectionComplete
			// Connection kann nicht in LHS enden
			if (getONTransition().getObjectNet() instanceof LHS) return null;
			MappingONTransitionCommand commandT = (MappingONTransitionCommand)command;
			// Target ONTransition setzen
			commandT.setTarget(getONTransition());
			// eventuell strende Mappings finden
			Mapping tempMapping= null;
			Mapping tempMapping2= null;
			// 1. Es gibt schon ein Mapping zwischen genau diesen 2 ONTransitions, dann dieses Mapping lschen.
			if ((tempMapping=thereIsAlreadyMapping(commandT)) != null) {
				DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
				deleteCommand.setMapping(tempMapping);
				return deleteCommand;
			// 2. Es gibt schon ein Mapping von der Source ONTransition aus der LHS zu irgendeinen anderen 
			// aus der Seite(RHS,NAC1,NAC2,...) dieser ONTransition.
			//Dann das alte Mapping lschen und dafr das neue einfgen
			} else if (((tempMapping= thereIsAlreadyMappingToSide(commandT)) != null)
					| ((tempMapping2= thereIsAlreadyMappingFromLHS(commandT)) != null)) {		
				CompoundCommand cmd= new CompoundCommand();
				if (tempMapping != null) {
					DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
					deleteCommand.setMapping(tempMapping);
					cmd.add(deleteCommand);
				}
				if (tempMapping2 != null) {
					DeleteMappingCommand deleteCommand = new DeleteMappingCommand();
					deleteCommand.setMapping(tempMapping2);
					cmd.add(deleteCommand);
				}
				cmd.add(commandT);
				return cmd;
			}			
		} else return null;
		return command;
	}
	
	protected ONTransition getONTransition() {
		return (ONTransition)getHost().getModel();
	}
	
	protected ONTransitionEditPart getONTransitionEditPart() {
		return (ONTransitionEditPart)getHost();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		Object reqObj = request.getNewObject();
		Command command = null;
		if (reqObj instanceof ONArc) {
			// ONArc Connection Create
			ConnectionONTransONPlaceCommand cmd = new ConnectionONTransONPlaceCommand();
			cmd.setONArc((ONArc)reqObj);
			cmd.setSource(getONTransition());
			ConnectionAnchor anchor = ((ONTransitionEditPart) getHost())
			.getSourceConnectionAnchor(request);
			if (anchor instanceof ONAnchor)
				cmd.setAnchorNrS(((ONAnchor)anchor).getId());
			else
				cmd.setAnchorNrS(-1);
			command= cmd;
		} else if (reqObj instanceof Mapping) {
			// Mapping ConnectionCreate
			// Mappings drfen nur in der LHS beginnen
			if (!(getONTransition().getObjectNet() instanceof LHS)) return null;
			MappingONTransitionCommand cmd = new MappingONTransitionCommand();
			// das neu erstellte Mapping ins Command tun
			cmd.setMapping((Mapping)reqObj);
			// aktuelle ONTransition ins Command tun
			cmd.setSource(getONTransition());
			command= cmd;
		}			
		request.setStartCommand(command);
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		ONArc onArc = (ONArc) request.getConnectionEditPart().getModel();		
		if (onArc.getTarget() instanceof ONPlace) {			
			return null;
		}	
		ReconnectONArcCommand command = new ReconnectONArcCommand(onArc);
		ConnectionAnchor anchor = ((ONTransitionEditPart) getHost())
				.getTargetConnectionAnchor(request);
		if (anchor instanceof ONAnchor)
			command.setNewAnchorNrT(((ONAnchor)anchor).getId());
		else
			command.setNewAnchorNrT(-1);
		command.setNewTarget(getONTransition());
		if (thereIsAlreadyONArc(command)) return null;
		if (nothingChanged(command)) return null;
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		ONArc onArc = (ONArc) request.getConnectionEditPart().getModel();
		if (onArc.getSource() instanceof ONPlace) {			
			return null;
		}
	ReconnectONArcCommand command = new ReconnectONArcCommand(onArc);
		ConnectionAnchor anchor = ((ONTransitionEditPart) getHost())
				.getSourceConnectionAnchor(request);
		if (anchor instanceof ONAnchor)
			command.setNewAnchorNrS(((ONAnchor)anchor).getId());
		else
			command.setNewAnchorNrS(-1);
		command.setNewSource(getONTransition());
		if (thereIsAlreadyONArc(command)) return null;
		if (nothingChanged(command)) return null;
		return command;
	}
}
