package roneditor.objectnetview.figures;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

import roneditor.objectnetview.anchors.ONAnchor;

/**
 * Represents the graphical layout of a place.
 */
public class ONTransitionFigure 
	extends AbstractONNodeFigure
{
	protected ChopboxAnchor incomingCBConnectionAnchor;
	protected ChopboxAnchor outgoingCBConnectionAnchor;
	//private FlowLayout layout;
	protected List<ONAnchor> incomingConnectionAnchors = new ArrayList<ONAnchor>();
	protected List<ONAnchor> outgoingConnectionAnchors = new ArrayList<ONAnchor>();
	protected TransitionFigure t;
	static public final int tSize = 25;
	
	public ONTransitionFigure() {

		this.setOutline(false);
//		layout = new FlowLayout(true);
//		layout.setMajorAlignment(0);
//		layout.setMajorSpacing(2);
//		this.setLayoutManager( layout);
		setLayoutManager(new XYLayout());
		this.setFill(false);
		
		t = new TransitionFigure();
		incomingCBConnectionAnchor = new ChopboxAnchor(t);
		outgoingCBConnectionAnchor = new ChopboxAnchor(t);
		t.createAnchors();
		t.setOutline(true);		
		t.setSize(tSize, tSize);
		t.setBackgroundColor(ColorConstants.white);
		t.setOpaque(false);
		this.add(t);

		mappingLabel = new Label("");
		mappingLabel.setForegroundColor(ColorConstants.blue);
		this.add(mappingLabel);


	}
	

	
	public void setColor(Color color){
		t.setBackgroundColor(color);
	}
	
	public void validate(){
		super.validate();
		
//		Dimension d = getPreferredSize(-1, -1);
		
//		this.setMaximumSize(d);
//		this.setMinimumSize(d);
//		this.setPreferredSize(d);
		getLayoutManager().setConstraint(t, new Rectangle(0, 0,
				tSize, tSize));
		getLayoutManager().setConstraint(mappingLabel, new Rectangle(tSize+3,1,-1,-1));
	}
	
	
	public ChopboxAnchor getSourceCBConnectionAnchor(){
		return outgoingCBConnectionAnchor;
	}
	public ChopboxAnchor getTargetCBConnectionAnchor(){
		return incomingCBConnectionAnchor;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {		
		return getPreferredSize();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	@Override
	public Dimension getPreferredSize(int wHint, int hHint) {
		if (mappingLabel.getSize().width > 0) {
			return new Dimension(tSize + mappingLabel.getSize().width+3, tSize);
		} else {
			return new Dimension(tSize, tSize);
		}
	}

	protected class TransitionFigure extends RectangleFigure{
		public TransitionFigure(){
			super();
			StackLayout l = new StackLayout();
			this.setLayoutManager(l);
			this.setFill(true);
			this.setMinimumSize(new Dimension(tSize,tSize));
			this.setPreferredSize(new Dimension(tSize,tSize));
			this.setMaximumSize(new Dimension(tSize,tSize));
		}
		
	
	/**
	 * This method creates the PAnchorPoints at the Node
	 */
	protected void createAnchors() {

		// 10 anchors around the transition figure
		Dimension dF = new Dimension(tSize,tSize);
		Point[] connectionPoints = new Point[32];

		connectionPoints[0] = new Point(dF.width * 1 / 8, 0);
		connectionPoints[12] = new Point(dF.width * 2 / 8, 0);
		connectionPoints[1] = new Point(dF.width * 3 / 8, 0);
		connectionPoints[13] = new Point(dF.width * 4 / 8, 0);
		connectionPoints[2] = new Point(dF.width * 5 / 8, 0);
		connectionPoints[14] = new Point(dF.width * 6 / 8, 0);
		connectionPoints[3] = new Point(dF.width * 7 / 8, 0);

		connectionPoints[15] = new Point(0, dF.height * 1 / 8);
		connectionPoints[16] = new Point(0, dF.height * 2 / 8);
		connectionPoints[4] = new Point(0, dF.height * 3 / 8);
		connectionPoints[17] = new Point(0, dF.height * 4 / 8);
		connectionPoints[6] = new Point(0, dF.height * 5 / 8);
		connectionPoints[18] = new Point(0, dF.height * 6 / 8);
		connectionPoints[19] = new Point(0, dF.height * 7 / 8);

		connectionPoints[20] = new Point(dF.width, dF.height * 1 / 8);
		connectionPoints[21] = new Point(dF.width, dF.height * 2 / 8);
		connectionPoints[5] = new Point(dF.width, dF.height * 3 / 8);
		connectionPoints[22] = new Point(dF.width, dF.height * 4 / 8);
		connectionPoints[7] = new Point(dF.width, dF.height * 5 / 8);
		connectionPoints[23] = new Point(dF.width, dF.height * 6 / 8);
		connectionPoints[24] = new Point(dF.width, dF.height * 7 / 8);

		connectionPoints[8] = new Point(dF.width * 1 / 8, dF.height);
		connectionPoints[25] = new Point(dF.width * 2 / 8, dF.height);
		connectionPoints[9] = new Point(dF.width * 3 / 8, dF.height);
		connectionPoints[26] = new Point(dF.width * 4 / 8, dF.height);
		connectionPoints[10] = new Point(dF.width * 5 / 8, dF.height);
		connectionPoints[27] = new Point(dF.width * 6 / 8, dF.height);
		connectionPoints[11] = new Point(dF.width * 7 / 8, dF.height);
		
		
		// die 4 Ecken
		connectionPoints[28] = new Point(0, 0);
		connectionPoints[29] = new Point(0, dF.height);
		connectionPoints[30] = new Point(dF.width , 0);
		connectionPoints[31] = new Point(dF.width, dF.height);

		ONAnchor tempA;
		for (int i = 0; i < 32; i++) {
			tempA = new ONAnchor(t);
			tempA.setRelativeDist(connectionPoints[i]);
			tempA.setId(i);
			incomingConnectionAnchors.add(tempA);
			outgoingConnectionAnchors.add(tempA);

		}
	}

	/**
	 * This method refreshes the Position of the AnchorPoints at the Node as
	 * soon as there has been occured a changing of the dimension
	 */
	protected void refreshAnchors() {

		Dimension dF = getPreferredSize(-1, -1);
		Point[] connectionPoints = new Point[32];

		connectionPoints[0] = new Point(dF.width * 1 / 8, 0);
		connectionPoints[12] = new Point(dF.width * 2 / 8, 0);
		connectionPoints[1] = new Point(dF.width * 3 / 8, 0);
		connectionPoints[13] = new Point(dF.width * 4 / 8, 0);
		connectionPoints[2] = new Point(dF.width * 5 / 8, 0);
		connectionPoints[14] = new Point(dF.width * 6 / 8, 0);
		connectionPoints[3] = new Point(dF.width * 7 / 8, 0);

		connectionPoints[15] = new Point(0, dF.height * 1 / 8);
		connectionPoints[16] = new Point(0, dF.height * 2 / 8);
		connectionPoints[4] = new Point(0, dF.height * 3 / 8);
		connectionPoints[17] = new Point(0, dF.height * 4 / 8);
		connectionPoints[6] = new Point(0, dF.height * 5 / 8);
		connectionPoints[18] = new Point(0, dF.height * 6 / 8);
		connectionPoints[19] = new Point(0, dF.height * 7 / 8);

		connectionPoints[20] = new Point(dF.width, dF.height * 1 / 8);
		connectionPoints[21] = new Point(dF.width, dF.height * 2 / 8);
		connectionPoints[5] = new Point(dF.width, dF.height * 3 / 8);
		connectionPoints[22] = new Point(dF.width, dF.height * 4 / 8);
		connectionPoints[7] = new Point(dF.width, dF.height * 5 / 8);
		connectionPoints[23] = new Point(dF.width, dF.height * 6 / 8);
		connectionPoints[24] = new Point(dF.width, dF.height * 7 / 8);

		connectionPoints[8] = new Point(dF.width * 1 / 8, dF.height);
		connectionPoints[25] = new Point(dF.width * 2 / 8, dF.height);
		connectionPoints[9] = new Point(dF.width * 3 / 8, dF.height);
		connectionPoints[26] = new Point(dF.width * 4 / 8, dF.height);
		connectionPoints[10] = new Point(dF.width * 5 / 8, dF.height);
		connectionPoints[27] = new Point(dF.width * 6 / 8, dF.height);
		connectionPoints[11] = new Point(dF.width * 7 / 8, dF.height);
		
		
		// die 4 Ecken
		connectionPoints[28] = new Point(0, 0);
		connectionPoints[29] = new Point(0, dF.height);
		connectionPoints[30] = new Point(dF.width , 0);
		connectionPoints[31] = new Point(dF.width, dF.height);

		for (int i = 0; i < 32; i++) {
			incomingConnectionAnchors.get(i).setRelativeDist(
					connectionPoints[i]);
			outgoingConnectionAnchors.get(i).setRelativeDist(
					connectionPoints[i]);
		}
	  }
	}

	/**
	 * Returns a list of anchors for source connections
	 * 
	 * @return
	 */
	public List<ONAnchor> getSourceConnectionAnchors() {
		return outgoingConnectionAnchors;
	}
	
	/**
	 * Returns a list of anchors for target connections.
	 * 
	 * @return
	 */
	public List<ONAnchor> getTargetConnectionAnchors() {
		return incomingConnectionAnchors;
	}
	
	/**
	 * Detection of the closest source connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getSourceConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getSourceConnectionAnchors(),getSourceCBConnectionAnchor());
	}

	/**
	 * Detection of the closest target connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getTargetConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getTargetConnectionAnchors(),getTargetCBConnectionAnchor());
	}

	/**
	 * Calculating the minimal distant anchor regarding to the mouse pointer
	 * 
	 * @param p
	 * @param connectionAnchors
	 * @return the closest anchor around the point p
	 */
	static public ConnectionAnchor getClosestConnectionAnchorAt(Point p,
			List<ONAnchor> connectionAnchors, ChopboxAnchor cba) {

		// initializing variables
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;
		int anz_Anchors = connectionAnchors.size();
		Point p2 = new Point(0, 0);

		// checking each anchor point and taking the best one
		for (int i = 0; i < anz_Anchors; i++) {
			ONAnchor a = connectionAnchors.get(i);

			// absolut coordinates of the anchor
			p2 = a.getLocation(null);

			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = a;
			}
		}
		
		//chopbox anchor
		p2 = cba.getReferencePoint();	
		long d = p.getDistance2(p2);
		if (d < min+50) {
			min = d;
			closest = cba;
		}
		return closest;
	}





}