package roneditor.ronview;

import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import roneditor.model.ModelCreationFactory;
import ronmodel.HLArc;
import ronmodel.HLPlaceType;
import ronmodel.HLTransitionKind;


/**
 * Main class for the objectnet editor palette. All graphical figures have to be
 * registered here.
 */
public class RONPaletteRoot extends PaletteRoot {

	/**
	 * Creates a new RONPaletteRoot instance.
	 * 
	 */
	public RONPaletteRoot() {
		// create root
		super();

		// a group of default control tools
		PaletteGroup controls = new PaletteGroup("Controls");
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols
		PaletteGroup symbols = new PaletteGroup("Symbols");
		add(symbols);
		
		PaletteDrawer hlPlace = new PaletteDrawer("HLPlace");
		PaletteDrawer hlTransition = new PaletteDrawer("HLTransition");
		
		hlPlace.setInitialState(PaletteDrawer.INITIAL_STATE_OPEN);
		hlTransition.setInitialState(PaletteDrawer.INITIAL_STATE_OPEN);

		CreationToolEntry entry;

		entry = new RONCreationToolEntry("HLPlace (Net)", "Creates an HLPlace of type:net",
				new ModelCreationFactory(HLPlaceType.NET_LITERAL), null, null);
		hlPlace.add(entry);

		entry = new RONCreationToolEntry("HLPlace (Rule)", "Creates an HLPlace of type:rule",
				new ModelCreationFactory(HLPlaceType.RULE_LITERAL), null, null);
		hlPlace.add(entry);
		
		symbols.add(hlPlace);
		
		entry = new RONCreationToolEntry("HLTransition (standard)",
				"Creates an HLTransition", new ModelCreationFactory(
						HLTransitionKind.STANDARD_LITERAL), null, null);
		hlTransition.add(entry);
		
		entry = new RONCreationToolEntry("HLTransition (apply rule)",
				"Creates an HLTransition", new ModelCreationFactory(
						HLTransitionKind.APPLYRULE_LITERAL), null, null);
		hlTransition.add(entry);
		
		entry = new RONCreationToolEntry("HLTransition (fire)",
				"Creates an HLTransition", new ModelCreationFactory(
						HLTransitionKind.FIRE_LITERAL), null, null);
		hlTransition.add(entry);
		
		entry = new RONCreationToolEntry("HLTransition (split)",
				"Creates an HLTransition", new ModelCreationFactory(
						HLTransitionKind.SPLIT_LITERAL), null, null);
		hlTransition.add(entry);
		
		symbols.add(hlTransition);

		entry = new RONConnectionCreationToolEntry("HLArc",
				"Creates an HLArc", new ModelCreationFactory(HLArc.class),
				null, null);
		//entry.setSmallIcon(ImageDescriptor.createFromFile(location,"icons/connection.gif"));
		symbols.add(entry);

		

		
	}
}
