package roneditor.ronview.edit.parts;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.RelativeBendpoint;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import roneditor.objectnetview.figures.ONArcLabelFigure;
import roneditor.objectnetview.figures.ONArcLabelMidPointLocator;
import roneditor.ronview.edit.policies.ArcSelectionHandlesEditPolicy;
import roneditor.ronview.edit.policies.ConnectionBendPointEditPolicy;
import roneditor.ronview.edit.policies.HLArcEditPolicy;
import roneditor.ronview.edit.policies.HLArcEndpointEditPolicy;
import ronmodel.BendPoint;
import ronmodel.HLArc;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a onArc.
 */
public class HLArcEditPart extends AbstractConnectionEditPart {

	private Notifier target;

	private OnArcAdapter adapter;
	
	ONArcLabelFigure labelFigure;

	public HLArcEditPart(HLArc hlArc) {
		super();
		setModel(hlArc);
	}

	public HLArc getHLArc() {
		return (HLArc) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new HLArcEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new ConnectionBendPointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new HLArcEditPolicy());
	}

	@Override
	protected IFigure createFigure() {
		PolylineConnection c = (PolylineConnection) super.createFigure();
		// target decoration -->
		// arrow at the end of the line
		PolygonDecoration decoration = new PolygonDecoration();
		c.setTargetDecoration(decoration);


//		labelFigure = new ONArcLabelFigure();
//		Integer weight = ((HLArc)getModel()).getWeight();
//		String labelText = "";
//		if (weight > 1) {
//			labelText = weight.toString();
//		}
//		labelFigure.setWeight(labelText);
//		ONArcLabelMidPointLocator constraint = new ONArcLabelMidPointLocator(
//				labelText, new Point(0,0), c);
//		c.add(labelFigure, constraint);
		c.setConnectionRouter(new BendpointConnectionRouter());
		return c;
	}

	@Override
	public void activate() {
		adapter = new OnArcAdapter();
		getHLArc().eAdapters().add(adapter);
		super.activate();
	}

	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();		
//		Integer weight = ((HLArc)getModel()).getWeight();
//		String labelText = "";
//		if (weight > 1) {
//			labelText = weight.toString();
//		}
//		labelFigure.setWeight(labelText);

		refreshBendpoints();
	}

	protected void refreshBendpoints() {
		List figureConstraint = new ArrayList();
		for (Object modelConstraint : getHLArc().getBendpoints()) {
			BendPoint wbp = (BendPoint) modelConstraint;
			// RelativeBendpoint rbp = new
			// RelativeBendpoint(getConnectionFigure());
			RelativeBendpoint rbp = new RelativeBendpoint(
					(Connection) getConnectionFigure());

			Dimension first = new Dimension((Integer) wbp.getW().get(0),
					(Integer) wbp.getH().get(0));
			Dimension second = new Dimension((Integer) wbp.getW().get(1),
					(Integer) wbp.getH().get(1));
			rbp.setRelativeDimensions(first, second);
			figureConstraint.add(rbp);
		}
		getConnectionFigure().setRoutingConstraint(figureConstraint);
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class OnArcAdapter implements Adapter {
		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.HL_ARC__ANCHOR_NR_S:
				refreshSourceAnchor();
				break;
			case RonmodelPackage.HL_ARC__ANCHOR_NR_T:
				refreshTargetAnchor();
				break;
			case RonmodelPackage.HL_ARC__BENDPOINTS:
				refreshBendpoints();
				break;
			case RonmodelPackage.HL_ARC__SOURCE:
			case RonmodelPackage.HL_ARC__TARGET:
			case RonmodelPackage.HL_ARC__RON:
				refreshVisuals();
//				try{
//					IResource res=(IResource) ((MultiViewerPageViewer)getViewer()).getHostPage().getEditor().getAdapter(IResource.class);
//					MappingConsistencyCheck.checkMappingConsistency(((HLArc)getModel()).getRON(),res);
//				} catch (NullPointerException npe){/*BH*/}
				break;
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(getModel().getClass());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class
}
