package roneditor.ronview.edit.parts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.IRonEditorConstants;
import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.edit.policies.TokenComponentEditPolicy;
import roneditor.editor.RONEditor;
import roneditor.objectnetview.model.EObjectPropertySource;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;
import ronmodel.Token;


/**
 * @author conny und alex
 * 
 * EditPart for a HLTransition. 
 */
public class HLTokenEditPart
	extends AbstractGraphicalEditPart
	implements Adapter
{
	private IPropertySource propertySource = null;
	private Notifier target;
	private Ellipse figure;
	
	
	/**
	 * Creates a new HLTransitionEditPart instance.
	 * @param element
	 */
	public HLTokenEditPart(Token token)
	{
		setModel(token);
	}

	public Token getToken() {
		return (Token)getModel();
	}

	
	public void notifyChanged(Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		Object obj = notification.getNotifier();

		if (obj instanceof Token) {
			switch (featureId) {
			case RonmodelPackage.TOKEN__NAME:
			case RonmodelPackage.TOKEN__X:
			case RonmodelPackage.TOKEN__Y:
			case RonmodelPackage.TOKEN:
				this.refreshVisuals();
				break;
			}
		}
		else  System.out.println(featureId);
	}


	/**
	 *@author conny 
	 * 
	 * this methode refreshs the figure of the Token
	 * 
	 * if the tokenEditPart is selected the background color is blue,
	 * else it is black
	 * 
	 * the tooltip of the token figure is the token name
	 * 
	 */
	@Override
	protected void refreshVisuals() {
		if(this.getSelected() == SELECTED_PRIMARY){
			this.figure.setBackgroundColor(ColorConstants.blue);
		}else{
			this.figure.setBackgroundColor(ColorConstants.black);
		}
		this.figure.setToolTip(new Label(((Token)getModel()).getName()));
		super.refreshVisuals();
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate()
	{
		if (isActive()){
			return;
		}

		// start listening for changes in the model
		
		((Notifier)getToken()).eAdapters().add(this);
		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier)getToken()).eAdapters().remove(this);

		super.deactivate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getToken() );
		}
		return propertySource;

	}

	/**
	 * @author conny und alex
	 * 
	 * this methode create the figure of this Token
	 * 
	 */
	@Override
	protected IFigure createFigure() {	
		figure=new Ellipse();
		figure.setSize(9, 9);
		figure.setBackgroundColor(ColorConstants.black);
		figure.setOpaque(true);
		figure.setToolTip(new Label(((Token)getModel()).getName()));
		return figure;
	}

	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new TokenComponentEditPolicy());	
	}
	
	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		if (req.getType().equals(RequestConstants.REQ_OPEN)) {
			//getParent().performRequest(req);
			openONView();
		}
	}
	
	/**
	 * @author alex
	 * 
	 * Opens a View for editing the object net that is this editpart's model.
	 */
	private void openONView() {
		if (getModel() instanceof ObjectNet) {
			((RONEditor)((MultiViewerPageViewer)getViewer()).getHostPage().getEditor()).showView(IRonEditorConstants.objectNetViewID,(EObject)getModel());
		} else {
			((RONEditor)((MultiViewerPageViewer)getViewer()).getHostPage().getEditor()).showView(IRonEditorConstants.ruleViewID,(EObject)getModel());
			if (((Rule)getModel()).getNacs().size() == 0) {
				//((RONEditor)((MultiViewerPageViewer)getViewer()).getHostPage().getEditor()).getSite().getPage().switchViewerVisibility(0);
			}
		}
	}
	
	
	/**
	 * @author conny
	 * 
	 * this methode calls refreshVisuals() of this Token, when its selection value has changed
	 * 
	 */
	@Override
	public void setSelected(int value) {
		super.setSelected(value);
		refreshVisuals();
	}	

}
