package roneditor.ronview.edit.parts;

import java.util.List;
import java.util.ListIterator;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.actions.FireTransitionAction;
import roneditor.editor.RONEditor;
import roneditor.objectnetview.model.EObjectPropertySource;
import roneditor.ronview.edit.policies.HLNodeComponentEditPolicy;
import roneditor.ronview.edit.policies.HLTransitionEditPolicy;
import roneditor.ronview.figures.HLTransitionFigure;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.RonmodelPackage;


/**
 * EditPart for a HLTransition. 
 */
public class HLTransitionEditPart 
	extends HLNodeEditPart
	implements NodeEditPart, Adapter
{
	private IPropertySource propertySource = null;
	private Notifier target;
	
	
	/**
	 * Creates a new HLTransitionEditPart instance.
	 * @param element
	 */
	public HLTransitionEditPart(HLTransition onTransition)
	{
		setModel(onTransition);
	}

	public HLTransition getHLTransition() {
		return (HLTransition)getModel();
	}

	

	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getHLTransition().getOutgoing();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getHLTransition().getIncoming();
	}
	

	/** 
	 * Returns the Figure of this, as a HLTransition type figure.
	 *
	 * @return  Figure as a HLTransitionFigure.
	 */
	protected HLTransitionFigure getHLTransitionFigure(){
		return (HLTransitionFigure) getFigure();
	}



	public void notifyChanged(Notification notification) {
		super.notifyChanged(notification);
		int featureId = notification.getFeatureID(RonmodelPackage.class );		
		switch( featureId ){ 
			case RonmodelPackage.HL_TRANSITION__OUTGOING:				
				refreshSourceConnections();	
				refreshVisuals();			
				break;		
			case RonmodelPackage.HL_TRANSITION__INCOMING:				
				refreshTargetConnections();					
				refreshVisuals();			
				break;
				case RonmodelPackage.HL_TRANSITION__NAME:
				case RonmodelPackage.HL_TRANSITION__X:
				case RonmodelPackage.HL_TRANSITION__Y:	
					this.refreshVisuals();	
					break;
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, new HLTransitionEditPolicy() );
		// install the edit policy to handle deletion
		installEditPolicy(
			EditPolicy.COMPONENT_ROLE,
			new HLNodeComponentEditPolicy());
	}
	
	/**
	 * refreshing the mouse point according to the request
	 * 
	 * @param point
	 */
	protected Point calculatePoint(Request req) {
		Point p = new Point(0, 0);
		if (req instanceof CreateConnectionRequest) {
			CreateConnectionRequest r = (CreateConnectionRequest) req;
			p = r.getLocation();
		} else if (req instanceof ReconnectRequest) {
			ReconnectRequest r2 = (ReconnectRequest) req;
			p = r2.getLocation();
		}
		return p;
	}// calculatePoint
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		ConnectionAnchor sourceAnchor;
		int anchorNr = ((HLArc) connection.getModel()).getAnchorNrS();
		if (anchorNr!=-1)
			sourceAnchor = getHLTransitionFigure().getSourceConnectionAnchors().get(anchorNr);
		else
			sourceAnchor = getHLTransitionFigure().getSourceCBConnectionAnchor();
		return sourceAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {		
		Point mousePoint = calculatePoint(request);		
		return getHLTransitionFigure().getSourceConnectionAnchorAt(mousePoint);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {		
		ConnectionAnchor targetAnchor;

		int anchorNr = ((HLArc) connection.getModel()).getAnchorNrT();			
		if (anchorNr!=-1)
			targetAnchor = getHLTransitionFigure().getTargetConnectionAnchors().get(anchorNr);
		else
			targetAnchor = getHLTransitionFigure().getTargetCBConnectionAnchor();
		return targetAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override	
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {		
		//return getHLTransitionFigure().getTargetConnectionAnchor( );
		Point mousePoint = calculatePoint(request);		
		return getHLTransitionFigure().getTargetConnectionAnchorAt(mousePoint); 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		Point loc = new Point( getHLTransition().getX(), getHLTransition().getY() );
		Dimension size = new Dimension(-1,-1);
		
		if(fireEnabled()){
			getHLTransitionFigure().setColor(ColorConstants.lightGreen);
		}
		else{
			getHLTransitionFigure().setColor(ColorConstants.red);
		}
		
		getHLTransitionFigure().setTransitionKind(((HLTransition)this.getModel()).getKind());
		
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);
	}
	
	/**
	 * @author Alex
	 * @return true if transition can fire, false if transition is not enabled or there is an error
	 * 	
	 */		
	public boolean fireEnabled(){		
		HLTransition trans=getHLTransition();
		List incoming = getHLTransition().getIncoming();
		ListIterator incomingIt = incoming.listIterator();
		List outgoing = getHLTransition().getOutgoing();
		ListIterator outgoingIt = outgoing.listIterator();
				
		//count pre-places
		int preRulePlaces = 0;
		int preNetPlaces = 0;		
		while(incomingIt.hasNext()){
			HLArc oa2 = (HLArc)incomingIt.next();
			HLPlace op2 = (HLPlace)oa2.getSource();
			if (op2==null) return false;
						
			if(op2.getType().getValue()==HLPlaceType.RULE)
				preRulePlaces++;
			else
				preNetPlaces++;
			
		}
		//count post-places
		int postRulePlaces = 0;
		int postNetPlaces = 0;		
		while(outgoingIt.hasNext()){
			HLArc oa2 = (HLArc)outgoingIt.next();
			HLPlace op2 = (HLPlace)oa2.getTarget();
			if (op2==null) return false;
						
			if(op2.getType().getValue()==HLPlaceType.RULE)
				postRulePlaces++;
			else
				postNetPlaces++;
			
		}
		
		//check pre- and post conditions
		if(trans.getKind()==HLTransitionKind.STANDARD_LITERAL){
			if (preNetPlaces==0 && postNetPlaces>0) return false;
			if (preRulePlaces>1) return false;
			if (preRulePlaces==0 && postRulePlaces>0) return false;			
		} else
		if(trans.getKind()==HLTransitionKind.SPLIT_LITERAL){
			if (preNetPlaces!=1) return false;
			if (preRulePlaces!=0) return false;			
			if (postNetPlaces!=1) return false;
			if (postRulePlaces!=0) return false;			
		} else
		if(trans.getKind()==HLTransitionKind.FIRE_LITERAL){
			if (preNetPlaces!=1) return false;
			if (preRulePlaces!=0) return false;
			if (postRulePlaces!=0) return false;
		} else
		if(trans.getKind()==HLTransitionKind.APPLYRULE_LITERAL){
			if (preNetPlaces<1) return false;
			if (preRulePlaces!=1) return false;
		}
		
		//count pre tokens and check for enough tokens
		incomingIt = incoming.listIterator();
		if(!incomingIt.hasNext()) return false;
		while (incomingIt.hasNext()){
			HLArc oa = (HLArc)incomingIt.next();
			if (oa == null) return false;	
			
			HLPlace op = (HLPlace)oa.getSource();			
			if (op == null) return false;	
			
			int preRuleToken = 0;
			int preONToken = 0;
			if (op!=null){
				if (op.getType().getValue()==HLPlaceType.RULE){
					preRuleToken=op.getTokens().size();
					if(trans.getKind()==HLTransitionKind.STANDARD_LITERAL){
						if(preRuleToken==0)return false;
					//hchstens 1 RULE-Stelle und beliebig viele ONs
					}
					if(trans.getKind()==HLTransitionKind.APPLYRULE_LITERAL){				
						if(preRuleToken==0)return false;
					//mind. 1 NET-Stelle, genau 1 RULE-Stelle			
					}
				}
				if (op.getType().getValue()==HLPlaceType.NET){
					preONToken=op.getTokens().size();
					if(trans.getKind()==HLTransitionKind.STANDARD_LITERAL){
						if(preONToken==0)return false;
					//hchstens 1 RULE-Stelle und beliebig viele ONs
					}
					if(trans.getKind()==HLTransitionKind.FIRE_LITERAL){
						if(preONToken==0)return false;
					// 1 NET-Stelle
					}
					if(trans.getKind()==HLTransitionKind.APPLYRULE_LITERAL){				
						if(preONToken==0)return false;
					//mind. 1 NET-Stelle, genau 1 RULE-Stelle			
					}
					if(trans.getKind()==HLTransitionKind.SPLIT_LITERAL){
						if(preONToken==0)return false;
					//eine NET-Stelle und keine RULE
					}
				}
			} else return false;
			
		}
		

		return true; 
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		
		((Notifier)getHLTransition()).eAdapters().add(this);

		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier)getHLTransition()).eAdapters().remove(this);

		super.deactivate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getHLTransition() );
		}
		return propertySource;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		HLTransitionKind kind = this.getHLTransition().getKind();
		HLTransitionFigure figure = new HLTransitionFigure();
		figure.setToolTip(new Label("Typ: "+kind.getName()));

		return figure;
	}
	
	
	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		if (req.getType().equals(RequestConstants.REQ_OPEN)) {
			//getParent().performRequest(req);
			FireTransitionAction fireAction = (FireTransitionAction)(((MultiViewerPageViewer)getViewer()).getHostPage()).getActionRegistry().getAction(FireTransitionAction.ID);
			if (fireAction.isEnabled()) {
				fireAction.run();
			}
		}
	}
	
	
	
}
