package roneditor.ronview.edit.policies;

import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editpolicies.ComponentEditPolicy;
import org.eclipse.gef.requests.GroupRequest;

import roneditor.commands.DeleteHLNodeCommand;
import roneditor.objectnetview.commands.DeleteMappingCommand;
import ronmodel.Codomain;
import ronmodel.HLNode;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.RHS;
import ronmodel.RON;
import ronmodel.Rule;

/**
 * EditPolicy for HLTransition component.
 */
public class HLNodeComponentEditPolicy extends ComponentEditPolicy {

	/**
	 * 
	 */
	public HLNodeComponentEditPolicy() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ComponentEditPolicy#createDeleteCommand(org.eclipse.gef.requests.GroupRequest)
	 */
	protected Command createDeleteCommand(GroupRequest deleteRequest) {
		HLNode onNode = (HLNode) getHost().getModel();
		RON objectNet = (RON) getHost().getParent().getModel();
		CompoundCommand result = new CompoundCommand();

		// Mappings suchen und diese dem CompoundCommand adden (mitlschen)
		if (objectNet instanceof Codomain) { // NAC oder RHS Knoten wird
												// gelscht
			List<Mapping> mappings = ((Codomain) objectNet).getMappings();
			for (Mapping mapping : mappings) {
				if (mapping.getTarget() == onNode) {
					// alle Mappings der Codomain durchgehen und nachsehen ob
					// die Source dem zu lschenden Knoten entspricht
					DeleteMappingCommand deleteMappingCommand = new DeleteMappingCommand();
					deleteMappingCommand.setMapping(mapping);
					result.add(deleteMappingCommand);
				}
			}
		} else if (objectNet instanceof LHS) { // LHS Knoten wird gelscht
			// Im gegensatz zu dem Fall das ein NAC/RHS Knoten gelscht wird,
			// kann in jeder Codomain ein Mapping gespeichert sein, das als
			// Source diesen HLNode hat.
			// Deswegen muss man alle RHS und NACs durchgehen.
			Rule rule = ((LHS) objectNet).getRule();
			RHS rhs = rule.getRhs();
			List<Mapping> mappingsRHS = rhs.getMappings();
			// alle Mappings der RHS durchgehen und nachsehen ob die Source dem
			// zu lschenden Knoten entspricht
			for (Mapping mapping : mappingsRHS) {
				if (mapping.getSource() == onNode) {
					DeleteMappingCommand deleteMappingCommand = new DeleteMappingCommand();
					deleteMappingCommand.setMapping(mapping);
					result.add(deleteMappingCommand);
				}
			}
			List<NAC> nacs = rule.getNacs();
			for (NAC nac : nacs) {
				// alle NACs durchgehen
				List<Mapping> mappingsNAC = nac.getMappings();
				for (Mapping mapping : mappingsNAC) {
					// alle Mappings der NAC durchgehen und nachsehen ob die
					// Source dem zu lschenden Knoten entspricht
					if (mapping.getSource() == onNode) {
						DeleteMappingCommand deleteMappingCommand = new DeleteMappingCommand();
						deleteMappingCommand.setMapping(mapping);
						result.add(deleteMappingCommand);
					}
				}
			}
		}

		// den HLNode lschen
		DeleteHLNodeCommand deleteCmd = new DeleteHLNodeCommand(
				(HLNode) getHost().getModel());

		result.add(deleteCmd);

		return result;
	}

}
