package roneditor.ronview.edit.policies;

import java.util.List;
import java.util.Vector;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.commands.AddHLTokenCommand;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * EditPolicy for XY-Layout on RONEditParts.
 * 
 * @author conny
 */
public class HLPlaceXYLayoutEditPolicy extends XYLayoutEditPolicy {



	/**
	 * @author conny 
	 * 
	 * methode for creating the addCommand for a given HLTokenEditPart
	 * 
	 * it checks consistency of types of this HLToken and the selected HLPlace
	 */
	@Override
	protected Command createAddCommand(final EditPart child, final Object constraint) {

			List<Token> tokens = new Vector<Token>();
		
			// beim Dragen eines HLPlace Labels kommt LabelEditPart als Child hier an
			// deswegen hier Abfangen und nur Tokens bearbeiten
			if (!(child.getModel() instanceof Token)) {
				return null;
			}
			
			Token token = (Token) child.getModel();
			HLPlace place = (HLPlace)getHost().getModel();
			
			switch(place.getType().getValue()){
				case HLPlaceType.RULE:
					if(token instanceof Rule){
						tokens.add(token);
						return new AddHLTokenCommand((HLPlace) getHost().getModel(), tokens);
					}break;
				
				case HLPlaceType.NET:
					if(token instanceof ObjectNet){
						tokens.add(token);
						return new AddHLTokenCommand((HLPlace) getHost().getModel(), tokens);
					}break;
			}
			return null;
	}
	

	/**
	 * @return A NonResizableEditPolicy to prevent the new EditPart being
	 *         resized by the user.
	 *         
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	@Override
	protected EditPolicy createChildEditPolicy(@SuppressWarnings("unused")
	final EditPart child) {
		return new NonResizableEditPolicy();
	}

	@Override
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		// TODO Auto-generated method stub
		return null;
	}
	
	@Override
	protected Command getCreateCommand(CreateRequest request) {
		// TODO Auto-generated method stub
		return null;
	}
	
}
