package roneditor.ronview.edit.policies;


import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.ronview.anchors.HLAnchor;
import roneditor.ronview.commands.ConnectionHLPlaceHLTransCommand;
import roneditor.ronview.commands.ConnectionHLTransHLPlaceCommand;
import roneditor.ronview.commands.ReconnectHLArcCommand;
import roneditor.ronview.edit.parts.HLTransitionEditPart;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;


/**
 * EditPolicy for a HLTransition.
 */
public class HLTransitionEditPolicy 
	extends HLNodeEditPolicy {


	public HLTransitionEditPolicy() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		ConnectionHLPlaceHLTransCommand commandPT;
		if (command instanceof ConnectionHLPlaceHLTransCommand) {
			// HLArc ConnectionComplete
			commandPT = (ConnectionHLPlaceHLTransCommand)command;	
			if (commandPT.getSource() == getHLTransition()) return null;		
			commandPT.setTarget(getHLTransition());
			if (thereIsAlreadyHLArc(commandPT)) return null;
			ConnectionAnchor anchor = ((HLTransitionEditPart) getHost())
			.getTargetConnectionAnchor(request);
			if (anchor instanceof HLAnchor)
				commandPT.setAnchorNrT(((HLAnchor)anchor).getId());
			else
				commandPT.setAnchorNrT(-1);		
		} else return null;
		if (!isAllowedArc(commandPT.getSource(), commandPT.getTarget(), false))
			return null;
		return command;
	}
	
	protected HLTransition getHLTransition() {
		return (HLTransition)getHost().getModel();
	}
	
	protected HLTransitionEditPart getHLTransitionEditPart() {
		return (HLTransitionEditPart)getHost();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		Object reqObj = request.getNewObject();
		Command command = null;
		if (reqObj instanceof HLArc) {
			// HLArc Connection Create
			ConnectionHLTransHLPlaceCommand cmd = new ConnectionHLTransHLPlaceCommand();
			cmd.setHLArc((HLArc)reqObj);
			cmd.setSource(getHLTransition());
			ConnectionAnchor anchor = ((HLTransitionEditPart) getHost())
			.getSourceConnectionAnchor(request);
			if (anchor instanceof HLAnchor)
				cmd.setAnchorNrS(((HLAnchor)anchor).getId());
			else
				cmd.setAnchorNrS(-1);
			command= cmd;
		}			
		request.setStartCommand(command);
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		HLArc onArc = (HLArc) request.getConnectionEditPart().getModel();		
		if (onArc.getTarget() instanceof HLPlace) {			
			return null;
		}	
		ReconnectHLArcCommand command = new ReconnectHLArcCommand(onArc);
		ConnectionAnchor anchor = ((HLTransitionEditPart) getHost())
				.getTargetConnectionAnchor(request);
		if (anchor instanceof HLAnchor)
			command.setNewAnchorNrT(((HLAnchor)anchor).getId());
		else
			command.setNewAnchorNrT(-1);
		command.setNewTarget(getHLTransition());
		if (thereIsAlreadyHLArc(command)) return null;
		if (nothingChanged(command)) return null;
		if (!isAllowedArc(onArc.getSource(),(HLTransition)getHost().getModel(),true))
			return null;
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		HLArc onArc = (HLArc) request.getConnectionEditPart().getModel();
		if (onArc.getSource() instanceof HLPlace) {			
			return null;
		}
	ReconnectHLArcCommand command = new ReconnectHLArcCommand(onArc);
		ConnectionAnchor anchor = ((HLTransitionEditPart) getHost())
				.getSourceConnectionAnchor(request);
		if (anchor instanceof HLAnchor)
			command.setNewAnchorNrS(((HLAnchor)anchor).getId());
		else
			command.setNewAnchorNrS(-1);
		command.setNewSource(getHLTransition());
		if (thereIsAlreadyHLArc(command)) return null;
		if (nothingChanged(command)) return null;
		if (!isAllowedArc((HLTransition)getHost().getModel(),onArc.getTarget(),true))		
			return null;
		return command;
	}
}
