package roneditor.ronview.figures;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

import roneditor.ronview.anchors.HLAnchor;

/**
 * Represents the graphical layout of a transition.
 */
public class HLPlaceFigure extends RectangleFigure
{
	static private int eSize = 50; //size of the place-ellipse (50) - NICHT kleiner als 20!!	

	protected EllipseAnchor incomingCBConnectionAnchor;
	protected EllipseAnchor outgoingCBConnectionAnchor;
	protected FlowLayout layout;
	protected List<HLAnchor> incomingConnectionAnchors = new ArrayList<HLAnchor>();
	protected List<HLAnchor> outgoingConnectionAnchors = new ArrayList<HLAnchor>();
	private EllipseFigure e;
	
	public HLPlaceFigure() {		
		setLayoutManager(new XYLayout());
		setOutline(false);
		setFill(false);
				
		
		e = new EllipseFigure();		
		e.setSize(eSize, eSize);
		e.setFill(false);
		e.setBackgroundColor(ColorConstants.white);
		e.setOpaque(false);
		e.setLineWidth(2);
		
		
		incomingCBConnectionAnchor = new EllipseAnchor(e);
		outgoingCBConnectionAnchor = new EllipseAnchor(e);
		createAnchors();
		
		this.add(e);
	}

	@Override
	public void validate(){		
		
		Dimension d = getPreferredSize(-1, -1);
		
		refreshAnchors();
		
		this.setMaximumSize(d);
		this.setMinimumSize(d);
		this.setPreferredSize(d);
		this.setOutline(false);

		// Die Ellipse fllt die Figure aus
		getLayoutManager().setConstraint(e, new Rectangle(0, 0,
					eSize, eSize));

		super.validate();
	}
	
	
	public EllipseAnchor getSourceCBConnectionAnchor(){
		return outgoingCBConnectionAnchor;
	}
	
	public EllipseAnchor getTargetCBConnectionAnchor(){
		return incomingCBConnectionAnchor;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	@Override
	protected boolean useLocalCoordinates() {
		return true;
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	@Override
	public Dimension getMinimumSize(int wHint, int hHint) {	
		return getPreferredSize();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	@Override
	public Dimension getPreferredSize(int wHint, int hHint) {
			return new Dimension(eSize, eSize);
	}

	/**
	 * This method creates the PAnchorPoints at the Node
	 */
	protected void createAnchors() {
		e.createAnchors();	
	}

	/**
	 * This method refreshes the Position of the AnchorPoints at the Node as
	 * soon as there has been occured a changing of the dimension
	 */
	protected void refreshAnchors() {
		e.refreshAnchors();
	}

	/**
	 * Returns a list of anchors for source connections
	 * 
	 * @return
	 */
	public List<HLAnchor> getSourceConnectionAnchors() {
		return outgoingConnectionAnchors;
	}
	
	/**
	 * Returns a list of anchors for target connections.
	 * 
	 * @return
	 */
	public List<HLAnchor> getTargetConnectionAnchors() {
		return incomingConnectionAnchors;
	}
	
	/**
	 * Detection of the closest source connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getSourceConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getSourceConnectionAnchors(),getSourceCBConnectionAnchor());
	}

	/**
	 * Detection of the closest target connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getTargetConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getTargetConnectionAnchors(),getTargetCBConnectionAnchor());
	}

	/**
	 * @author alex
	 * 
	 * Calculating the minimal distant anchor regarding to the mouse pointer
	 * 
	 * @param p
	 * @param connectionAnchors
	 * @return the closest anchor around the point p
	 */
	static private ConnectionAnchor getClosestConnectionAnchorAt(Point p,
			List<HLAnchor> connectionAnchors, EllipseAnchor cba) {

		// initializing variables
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;
		int anz_Anchors = connectionAnchors.size();
		Point p2 = new Point(0, 0);

		// checking each anchor point and taking the best one
		for (int i = 0; i < anz_Anchors; i++) {
			HLAnchor a = connectionAnchors.get(i);

			// absolut coordinates of the anchor
			p2 = a.getLocation(null);

			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = a;
			}
		}
		//chopbox anchor
		p2 = cba.getReferencePoint();
		long d = p.getDistance2(p2);
		if (d < min) {
			min = d;
			closest = cba;
		}
		return closest;
	}
	
	protected class EllipseFigure extends Ellipse{		
				
		public EllipseFigure(){
			super();
			XYLayout l = new XYLayout();
			this.setLayoutManager(l);
			this.setFill(true);
		}
		
		/**
		 * @author alex
		 * 
		 * This method creates the PAnchorPoints at the Node
		 */
		protected void createAnchors() {

			// 10 anchors around the transition figure
			//Dimension dF = new Dimension(eSize,eSize); //getPreferredSize(-1, -1);
			Point[] connectionPoints = new Point[12];
			
			
			Point relativerMittelPunkt = new Point(eSize/2,eSize/2);
			
			// rechts
			connectionPoints[0] = new Point(eSize/2*Math.cos(0),eSize/2*Math.sin(0));
			// oben
			connectionPoints[1] = new Point(eSize/2*Math.cos(Math.PI/2),eSize/2*Math.sin(Math.PI/2));
			// links
			connectionPoints[2] = new Point(eSize/2*Math.cos(Math.PI),eSize/2*Math.sin(Math.PI));
			// rechts
			connectionPoints[3] = new Point(eSize/2*Math.cos(Math.PI*3/2),eSize/2*Math.sin(Math.PI*3/2));
			
			
			// 2 weitere zwischen rechts und oben
			connectionPoints[4] = new Point(eSize/2*Math.cos(Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2*1/3));
			connectionPoints[5] = new Point(eSize/2*Math.cos(Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2*2/3));
			// 2 weitere zwischen oben und links
			connectionPoints[6] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*1/3));
			connectionPoints[7] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*2/3));
			// 2 weitere zwischen links und unten
			connectionPoints[8] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI+Math.PI/2*1/3));
			connectionPoints[9] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI+Math.PI/2*2/3));
			// 2 weitere zwischen unten und rechts
			connectionPoints[10] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*1/3));
			connectionPoints[11] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*2/3));
			

			HLAnchor tempA;
			for (int i = 0; i < 12; i++) {
				tempA = new HLAnchor(e);
				tempA.setRelativeDist(relativerMittelPunkt.getTranslated(connectionPoints[i]));
				tempA.setId(i);
				incomingConnectionAnchors.add(tempA);
				outgoingConnectionAnchors.add(tempA);

			}
		}

		/**
		 * @author alex
		 * 
		 * This method refreshes the Position of the AnchorPoints at the Node as
		 * soon as there has been occured a changing of the dimension
		 */
		protected void refreshAnchors() {

			//Dimension dF = getPreferredSize(-1, -1);
			Point[] connectionPoints = new Point[12];

			
			
			Point relativerMittelPunkt = new Point(eSize/2,eSize/2);
			
			// rechts
			connectionPoints[0] = new Point(eSize/2*Math.cos(0),eSize/2*Math.sin(0));
			// oben
			connectionPoints[1] = new Point(eSize/2*Math.cos(Math.PI/2),eSize/2*Math.sin(Math.PI/2));
			// links
			connectionPoints[2] = new Point(eSize/2*Math.cos(Math.PI),eSize/2*Math.sin(Math.PI));
			// rechts
			connectionPoints[3] = new Point(eSize/2*Math.cos(Math.PI*3/2),eSize/2*Math.sin(Math.PI*3/2));
			
			
			// 2 weitere zwischen rechts und oben
			connectionPoints[4] = new Point(eSize/2*Math.cos(Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2*1/3));
			connectionPoints[5] = new Point(eSize/2*Math.cos(Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2*2/3));
			// 2 weitere zwischen oben und links
			connectionPoints[6] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*1/3));
			connectionPoints[7] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*2/3));
			// 2 weitere zwischen links und unten
			connectionPoints[8] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI+Math.PI/2*1/3));
			connectionPoints[9] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI+Math.PI/2*2/3));
			// 2 weitere zwischen unten und rechts
			connectionPoints[10] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*1/3));
			connectionPoints[11] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*2/3));
			
			
			for (int i = 0; i < 12; i++) {
				incomingConnectionAnchors.get(i).setRelativeDist(relativerMittelPunkt.getTranslated(connectionPoints[i]));
				outgoingConnectionAnchors.get(i).setRelativeDist(relativerMittelPunkt.getTranslated(connectionPoints[i]));
			}
		}
		
	}
	
	
	/**
	 * @author conny
	 * 
	 * This Methode add a Token to the ellipse of its parent HLPlace-figure.
	 * 
	 * @param IFigure figure - Token figure to add
	 * @param int index - index of the current Token in children list
	 * @param int children - max number of children of all HLPlaces
	 * 
	 */
		public void addToE(IFigure figure, int index, int children){
			
//			number of Tokens per line
			int childrenPerLine = 2;
			if(children != 0 && children !=1){
				childrenPerLine = (int)(Math.ceil(Math.sqrt(children))); 
			}
			
//			size of the inner quad
			int quadSize =(childrenPerLine)*(figure.getSize().height+4); 
			
//			size of the HLPlace ellipse
			this.setESize((int)Math.sqrt((2*(Math.pow(quadSize, 2))))); 
			
//			center point of the ellipse
			int mPointXY = getESize()/2; 
			
//			left upper corner of the inner quad
			int begin = mPointXY-(quadSize/2) + 2; 
		
			if(childrenPerLine == 0){ children = 1;}
//			point of current Token - computation: modulo of the number of Tokens per line 
			figure.setLocation(new Point(begin+((figure.getSize().height+4)*(index%childrenPerLine)), 
										begin+((figure.getSize().height+4)*Math.ceil(index/childrenPerLine))));
			
			e.add(figure, index);
			
		}

		
		/**
		 * @author conny
		 * 
		 * this methode removes all token figures from the ellipse
		 * 
		 */
		public void removeAllChildrenFromE(){
			this.e.getChildren().clear();
		}
		
		/**
		 * @author conny
		 * 
		 * this methode removes one token figure from the ellipse
		 * 
		 */
		public void removeChildFromE(IFigure figure){
			this.e.remove(figure);
		}
		
		public static int getESize() {
			return eSize;
		}

		public void setESize(int size) {
			eSize = size;
		}

		public EllipseFigure getE() {
			return e;
		}

		public void setE(EllipseFigure e) {
			this.e = e;
		}
		
		public void setEclipseForegroundColor(Color color){
			e.setForegroundColor(color);
		}

	
}
