package roneditor.ruleview;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.actions.AddTokenAction;
import roneditor.editor.ContextMenuProviderWithActionRegistry;
import roneditor.objectnetview.actions.AutoLayoutAction;
import roneditor.objectnetview.actions.EditAction;
import roneditor.ronview.actions.DeleteHLBendPointAction;
import roneditor.ruleview.actions.CopyLhsToNacAction;
import roneditor.ruleview.actions.CopyLhsToRhsAction;
import roneditor.ruleview.actions.CopyRhsToNacAction;
import roneditor.ruleview.actions.CreateNacAction;
import roneditor.ruleview.actions.DeleteNacAction;
import roneditor.ruleview.actions.DeleteRuleBendPointAction;
import roneditor.ruleview.actions.SwitchNACAction;

/**
 * This is used to populate the context menu for the Viewers in the
 * {@link RuleNetPage}.
 * 
 * @author Tony Modica
 */
public class RulePageContextMenuProvider extends
		ContextMenuProviderWithActionRegistry {

	private final ActionRegistry actionRegistry;
	
	public RulePageContextMenuProvider(final EditPartViewer viewer,
			final ActionRegistry actionRegistry) {
		super(viewer, actionRegistry);
		this.actionRegistry = actionRegistry;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	public void buildContextMenu(final IMenuManager menuManager) {
		GEFActionConstants.addStandardActionGroups(menuManager);

		// Actions in UNDO group
		String group = GEFActionConstants.GROUP_UNDO;
		appendActionToGroup(menuManager, 
				ActionFactory.UNDO.getId(), group);
		appendActionToGroup(menuManager, 
				ActionFactory.REDO.getId(), group);

		// Actions in EDIT group
		group = GEFActionConstants.GROUP_EDIT;
		appendActionToGroup(menuManager, 
				ActionFactory.DELETE.getId(), group);
		dynamicAppendActionToGroup(menuManager, 
				GEFActionConstants.DIRECT_EDIT,group);
		dynamicAppendActionToGroup(menuManager, 
				EditAction.ID, group);
		appendActionToGroup(menuManager, 
				DeleteRuleBendPointAction.ID, group);
		dynamicAppendActionToGroup(menuManager, 
				AddTokenAction.ID, group);
		dynamicAppendActionToGroup(menuManager, 
				AutoLayoutAction.ID, group);
		
		
		dynamicAppendActionToGroup(menuManager, CopyLhsToRhsAction.ID, 
				GEFActionConstants.GROUP_COPY);
		dynamicAppendActionToGroup(menuManager, CopyLhsToNacAction.ID, 
				GEFActionConstants.GROUP_COPY);
		dynamicAppendActionToGroup(menuManager, CopyRhsToNacAction.ID, 
				GEFActionConstants.GROUP_COPY);

		// Sample Action to switch NACs
		dynamicAppendActionToGroup(menuManager, SwitchNACAction.ID,
				GEFActionConstants.GROUP_VIEW);
		// Create Nac
		dynamicAppendActionToGroup(menuManager, CreateNacAction.ID, 
				GEFActionConstants.GROUP_VIEW);
		// Delete Nac
		dynamicAppendActionToGroup(menuManager, DeleteNacAction.ID, 
				GEFActionConstants.GROUP_VIEW);
		

		

		

		
	}
}
