package roneditor.ruleview.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.model.ModelCreationFactory;
import roneditor.objectnetview.commands.MappingONPlaceCommand;
import roneditor.objectnetview.commands.MappingONTransitionCommand;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.Rule;

/**
 * @author conny
 * 
 * Command to copy the LHS to the current NAC. 
 * It is used by CopyLhsToNacAction.
 */

public class CopyLhsToNacCommand extends Command {
	private Rule rule;
	private EObject viewerContent;
	private NAC currentNac;
	private List lhsArcs, lhsNodes;
	private CompoundCommand cCommand;

	public CopyLhsToNacCommand() {
		setLabel("Copy LHS To NAC");
	}

	public void execute() {
//		copy of lhs
		LHS copy = (LHS) EcoreUtil.copy(this.rule.getLhs());
		
		
//		 get the current nac
		final int current = this.rule.getNacs().lastIndexOf(viewerContent);
		this.currentNac = (NAC) this.rule.getNacs().get(current);
		
		// get the nodes and arcs of the lhs copy
		this.lhsNodes = new ArrayList<Object>(copy.getNodes());
		this.lhsArcs = new ArrayList<Object>(copy.getArcs());
		this.cCommand = new CompoundCommand();
		
		
		
		// mappings nur erstellen, wenn auf der Zielseite noch keine sind, sonst kann es zu Konflikten kommen
		if (this.currentNac.getMappings().isEmpty()){ 	
	//		add nodes and arcs of the lhs copy to the current nac
			EList orgLhsNodes = this.rule.getLhs().getNodes();
			ModelCreationFactory mappingFactory = new ModelCreationFactory(Mapping.class);
			// Mapping hinzufgen		
			for (int i = 0; i < orgLhsNodes.size(); i++) {
				ONNode copyLhsNode = (ONNode)lhsNodes.get(i);
				this.currentNac.getNodes().add(lhsNodes.get(i));
				
				Mapping newMapping= (Mapping)mappingFactory.getNewObject();
				if (copyLhsNode instanceof ONPlace) {
					MappingONPlaceCommand command = new MappingONPlaceCommand();
					command.setMapping(newMapping);
					command.setSource((ONPlace)orgLhsNodes.get(i));
					command.setTarget((ONPlace)copyLhsNode);
					cCommand.add(command);					
				} else {
					MappingONTransitionCommand command = new MappingONTransitionCommand();
					command.setMapping(newMapping);
					command.setSource((ONTransition)orgLhsNodes.get(i));
					command.setTarget((ONTransition)copyLhsNode);	
					cCommand.add(command);				
				}
			}
			cCommand.execute();
		} else {
			// wenn keine Mappings eingefgt werden, einfach alle Knoten einfgen
			this.currentNac.getNodes().addAll(this.lhsNodes);			
		}
		this.currentNac.getArcs().addAll(this.lhsArcs);
		copy = null;
	}


	public void undo() {
		//remove the arcs and nodes of LHS from the current NAC
		this.currentNac.getArcs().removeAll(this.lhsArcs);
		this.currentNac.getNodes().removeAll(this.lhsNodes);
		
		cCommand.undo();

	}

	/*
	 *  add nodes and arcs of the lhs copy to the current nac
	 */
	public void redo() {
		this.currentNac.getNodes().addAll(this.lhsNodes);
		this.currentNac.getArcs().addAll(this.lhsArcs);
		
		cCommand.redo();
	}

	public void setRule(Rule _rule) {
		this.rule = _rule;
	}

	public void setViewerContent(EObject _content) {
		this.viewerContent = _content;
	}

}
