package roneditor.ruleview.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.model.ModelCreationFactory;
import roneditor.objectnetview.commands.MappingONPlaceCommand;
import roneditor.objectnetview.commands.MappingONTransitionCommand;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RHS;
import ronmodel.Rule;

/**
 * @author conny
 * 
 * Command to copy the RHS to the current NAC. 
 * It is used by CopyRhsToNacAction.
 */

public class CopyRhsToNacCommand extends Command {
	private Rule rule;
	private EObject viewerContent;
	private NAC currentNac;
	private List rhsArcs, rhsNodes;
	private CompoundCommand cCommand;

	public CopyRhsToNacCommand() {
		setLabel("Copy RHS To NAC");
	}

	public void execute() {

		// copy of rhs
		RHS copy = (RHS) EcoreUtil.copy(this.rule.getRhs());
		// alte Mappings lschen, die funktionieren nicht (Copy schafft das nicht)
		copy.getMappings().clear();
		for (Object o:copy.getNodes()) {
			List tempList= null;
			if ((tempList = ((ONNode)o).getIncomingM()) != null) {
				tempList.clear();
			}
			if ((tempList = ((ONNode)o).getOutgoingM()) != null) {
				tempList.clear();
			}
		}
		
		// get the current nac
		final int current = this.rule.getNacs().lastIndexOf(viewerContent);
		this.currentNac = (NAC) this.rule.getNacs().get(current);

		// get the nodes and arcs of the rhs
		this.rhsNodes = new ArrayList<Object>(copy.getNodes());
		this.rhsArcs = new ArrayList<Object>(copy.getArcs());

		// add nodes and arcs of the rhs copy to the current nac

		// create List for new Mappings
		this.cCommand = new CompoundCommand();
		
		
		// mappings nur erstellen, wenn auf der Zielseite noch keine sind, sonst kann es zu Konflikten kommen
		if (this.currentNac.getMappings().isEmpty()){ 		
			ModelCreationFactory mappingFactory = new ModelCreationFactory(Mapping.class);
			// Mapping hinzufgen		
			for (int i = 0; i < this.rhsNodes.size(); i++) {
				ONNode copyRhsNode = (ONNode)rhsNodes.get(i);
				this.currentNac.getNodes().add(rhsNodes.get(i));
				EList mappings = ((ONNode)rule.getRhs().getNodes().get(i)).getIncomingM();
				for (int j = 0; j < mappings.size(); j++) {								
					Mapping newMapping= (Mapping)mappingFactory.getNewObject();
					if (copyRhsNode instanceof ONPlace) {
						MappingONPlaceCommand command = new MappingONPlaceCommand();
						command.setMapping(newMapping);
						command.setSource((ONPlace)((Mapping)mappings.get(j)).getSource());
						command.setTarget((ONPlace)copyRhsNode);
						cCommand.add(command);					
					} else {
						MappingONTransitionCommand command = new MappingONTransitionCommand();
						command.setMapping(newMapping);
						command.setSource((ONTransition)((Mapping)mappings.get(j)).getSource());
						command.setTarget((ONTransition)copyRhsNode);	
						cCommand.add(command);				
					}
				}	
			}
			cCommand.execute();
		} else {
			// wenn keine Mappings eingefgt werden, einfach alle Knoten einfgen
			this.currentNac.getNodes().addAll(this.rhsNodes);			
		}
		this.currentNac.getArcs().addAll(this.rhsArcs);
		copy = null;
	}


	public void undo() {
		//remove the arcs and nodes of RHS from the current NAC
		this.currentNac.getArcs().removeAll(this.rhsArcs);
		this.currentNac.getNodes().removeAll(this.rhsNodes);
		cCommand.undo();

	}

	/*
	 *  add nodes and arcs of the rhs copy to the current nac
	 */
	public void redo() {
		this.currentNac.getNodes().addAll(this.rhsNodes);
		this.currentNac.getArcs().addAll(this.rhsArcs);
		cCommand.redo();
	}

	public void setRule(Rule _rule) {
		this.rule = _rule;
	}

	public void setViewerContent(EObject _content) {
		this.viewerContent = _content;
	}

}
